package edu.wustl.cse.mobilab.openworkflow.auction;

import java.util.SortedSet;

public class TaskBidComparatorImpl implements TaskBidComparator {

    /**
     * Compares two TaskBids on the basis of their capability fraction.
     * In CiAN velocity was used to break ties, but no longer.
     */
    public int compare(TaskBid leftTaskBid, TaskBid rightTaskBid) {
        int nRelation=Double.compare(leftTaskBid.getCapabilityFraction(), rightTaskBid.getCapabilityFraction());
        if (0==nRelation) {
            // Break ties in capability fraction
            //dLeftValue=(leftTaskBid.getPrecedingVelocity()+leftTaskBid.getSucceedingVelocity())/(2.0*leftTaskBid.getMaxVelocity());
            //dRightValue=(rightTaskBid.getPrecedingVelocity()+rightTaskBid.getSucceedingVelocity())/(2.0*rightTaskBid.getMaxVelocity());
            nRelation=leftTaskBid.getHostId().toString().compareTo(rightTaskBid.getHostId().toString());
        }
        return nRelation;
    }

    /**
     * This implementation strictly prefers non-conflicting bids to those
     * with conflicts. Next, we consider bids with conflicts, none of which
     * are temporarily allocated. If every bid has at least one conflict
     * which has been allocated, we return null here.
     */
    public TaskBid selectBestBid(SortedSet<TaskBid> taskBids) {
        TaskBid zeroConflictTaskBid=selectFirstNonConflict(taskBids);
        if (null!=zeroConflictTaskBid) {
            return zeroConflictTaskBid;
        }

        TaskBid resultTaskBid=null;
        for (TaskBid taskBid : taskBids) {
            boolean conflictScheduled=false;
            //for (TaskBid conflict : bidConflicts.get(taskBid)) {
            //    if (alloc.containsValue(conflict)) {
            //        conflictScheduled=true;
            //        break;
            //    }
            //}
            if (!conflictScheduled) {
                resultTaskBid=taskBid;
                break;
            }
        }
        return resultTaskBid;
    }

    /**
     * The old method from CiAN which removes the best ranked bid with no
     * conflicts. The ranking comes from the taskBidComparator. This method
     * is now used a subroutine.
     */
    private TaskBid selectFirstNonConflict(SortedSet<TaskBid> taskBids) {
        TaskBid resultTaskBid=null;
        for (TaskBid taskBid : taskBids) {
            //if (bidConflicts.get(taskBid).isEmpty()) {
            resultTaskBid=taskBid;
            break;
            //}
        }
        return resultTaskBid;
    }
}
