package edu.wustl.cse.mobilab.openworkflow.auction;

import edu.wustl.cse.mobilab.openworkflow.comm.HostId;
import edu.wustl.cse.mobilab.openworkflow.schedule.Time;
import edu.wustl.cse.mobilab.openworkflow.workflow.Task;

/**
 * Represents a worker's bid on a task.
 * This implementation assumes that every TaskBid given out by a worker
 * can be allocated simultaneously (there are no conflicts among outstanding
 * TaskBids).
 */
public class TaskBidImpl implements TaskBid {

    private final double m_dCapabilityFraction;
    private final Time m_deadlineTime;
    private final HostId m_hostId;
    private final Task m_task;

    public TaskBidImpl(Task task, HostId hostId, double dCapabilityFraction, Time deadlineTime) {
        m_task=task;
        m_hostId=hostId;
        m_dCapabilityFraction=dCapabilityFraction;
        m_deadlineTime=deadlineTime;
    }

    public double getCapabilityFraction() {
        return m_dCapabilityFraction;
    }

    public Time getDeadline() {
        return m_deadlineTime;
    }

    public HostId getHostId() {
        return m_hostId;
    }

    public Task getTask() {
        return m_task;
    }

    @Override
    public String toString() {
        return "{TaskBid task:"+m_task+" hostId:"+m_hostId+" capabilityFraction:"+m_dCapabilityFraction+" deadline:"+m_deadlineTime+"]";
    }
}
