/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package edu.wustl.cse.mobilab.openworkflow.comm;

import edu.wustl.cse.mobilab.openworkflow.shell.Disposable;
import edu.wustl.cse.mobilab.openworkflow.verify.Assert;
import edu.wustl.cse.mobilab.openworkflow.workflow.CompositeWorkflow;
import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNode;
import edu.wustl.cse.mobilab.openworkflow.workflow.TaskNode;
import lights.adapters.Tuple;
import lime.LimeException;
import lime.LimeTupleSpace;
import lime.StationaryAgent;
import lime.TupleSpaceEngineException;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

/**
 *
 * @author wilsonj
 */
public class FragmentObserver extends StationaryAgent implements FragmentObservable.Observer, Disposable {

    private static final long serialVersionUID = 1;

    private LimeTupleSpace m_limeTupleSpace;
    private long m_nextWorkflowID = 0;
    private Map<CompositeWorkflow, Long> m_workflowMap = new HashMap<CompositeWorkflow, Long>();
    private final BlockingQueue<Runnable> m_runnables=new LinkedBlockingQueue<Runnable>();

    private static final Runnable SHUTDOWN=new Runnable() {
        public void run() {
            Assert.statementNeverExecuted();
        }
    };

    //----------------------------------------------------------------
    public void update(final CompositeWorkflow oldWorkflow, final CompositeWorkflow newWorkflow) {
        m_runnables.add(new Runnable() {
            public void run() {
                if (oldWorkflow == null && newWorkflow != null) {
                    /* Add the CompositeWorkflow. */
                    addWorkflow(newWorkflow);
                } else if (oldWorkflow != null && newWorkflow == null) {
                    /* Remove the CompositeWorkflow. */
                    removeWorkflow(oldWorkflow);
                } else if (oldWorkflow != null && newWorkflow != null) {
                    /* Update the workflow. */
                    removeWorkflow(oldWorkflow);
                    addWorkflow(newWorkflow);
                }
            }
        });
    }

    //----------------------------------------------------------------
    @Override
    public void run() {

        try {
            /* Get a reference to the tuple space. */
            m_limeTupleSpace=new LimeTupleSpace("OpenWorkflow");
            m_limeTupleSpace.setShared(true);

            while (true) {
                Runnable runnable=null;
                try {
                    runnable=m_runnables.take();
                } catch (InterruptedException e) {
                    Assert.exceptionNeverCaught(e);
                }
                if (SHUTDOWN==runnable) {
                    break;
                }
                runnable.run();
            }


        } catch (LimeException le) {
            Assert.exceptionNeverCaught(le);
        }

    }

    //----------------------------------------------------------------
    public void dispose() {
        m_runnables.add(SHUTDOWN);
    }

    //----------------------------------------------------------------
    private void addWorkflow(CompositeWorkflow newWorkflow) {
        try {
            /* New workflow that needs to be added. */

            /* Insert a tuple for the composite workflow.
                * We do this first because inserting the ConfigurationNodes
                * could trigger reactions that need this tuple.
                */

            m_limeTupleSpace.out(new Tuple().addActual(m_nextWorkflowID).addActual(new CompositeWorkflowWrapper(newWorkflow)));
            m_workflowMap.put(newWorkflow, m_nextWorkflowID);
            /* Insert tuples for each ConfigurationNode in the CompositeWorkflow. */
            for (TaskNode taskNode : newWorkflow.getTaskNodes()) {
                for (ConfigurationNode node : taskNode.getPreconditions()) {
                    m_limeTupleSpace.out(new Tuple().addActual(new ConfigurationNodeWrapper(node)).addActual(m_nextWorkflowID));
                }
                for (ConfigurationNode node : taskNode.getPostconditions()) {
                    m_limeTupleSpace.out(new Tuple().addActual(new ConfigurationNodeWrapper(node)).addActual(m_nextWorkflowID));
                }
            }
            /* Update the id for the next CompositeWorkflow. */
            /* TODO: Fix overflow condition.  Will require sets of interval. */
            ++m_nextWorkflowID;
        } catch (TupleSpaceEngineException e) {
            Assert.exceptionNeverCaught(e);
        }
    }

    //----------------------------------------------------------------
    private void removeWorkflow(CompositeWorkflow oldWorkflow) {
        try {
            /* Old workflow that needs to be removed. */

            Long id = m_workflowMap.remove(oldWorkflow);
            /* Remove tuples for each ConfigurationNode in the CompositeWorkflow. */
            for (TaskNode taskNode : oldWorkflow.getTaskNodes()) {
                for (ConfigurationNode node : taskNode.getPreconditions()) {
                    m_limeTupleSpace.in(new Tuple().addActual(new ConfigurationNodeWrapper(node)).addActual(id));
                }
                for (ConfigurationNode node : taskNode.getPostconditions()) {
                    m_limeTupleSpace.in(new Tuple().addActual(new ConfigurationNodeWrapper(node)).addActual(id));
                }
            }
            /* Remove the CompositeWorkflow. */
            m_limeTupleSpace.in(new Tuple().addActual(id).addActual(new CompositeWorkflowWrapper(oldWorkflow)));
        } catch (TupleSpaceEngineException e) {
            Assert.exceptionNeverCaught(e);
        }
    }
}