package edu.wustl.cse.mobilab.openworkflow.comm;

import edu.wustl.cse.mobilab.openworkflow.workflow.CompositeWorkflow;
import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNode;

//--------------------------------------------------------------------
/** Can answer queries about the existence of workflow fragments
 * containing a given configuration node.
 * @author Created by Louis Thomas on Nov 6, 2008. */
public interface FragmentQueryable {

    /** Starts an open-ended query for any fragments containing the
     * given configuration node. Requests that the given listener be
     * called back with any matching fragments. Callbacks may be made
     * at any time until the listener is unsubscribed. It is an error
     * to subscribe twice with a given {ConfigurationNode, Listener}
     * pair without unsubscribing. */
    void subscribeFragmentsWithConfiguration(ConfigurationNode configurationNode, Listener listener);

    /** Stops an open-ended query for any fragments containing the
     * given configuration node. It is an error to unsubscribe from
     * a given {ConfigurationNode, Listener} pair without first
     * subscribing. */
    void unsubscribeFragmentsWithConfiguration(ConfigurationNode configurationNode, Listener listener);

    interface Listener {
        /** Notifies this Listener that a matching fragment has been
         * found. */
        void handleFragment(CompositeWorkflow compositeWorkflow);
    }
}
