/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package edu.wustl.cse.mobilab.openworkflow.comm;

import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNode;
import lime.LimeException;
import lime.LimeServer;
import org.apache.log4j.Logger;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Executor;

/**
 *
 * @author wilsonj
 */
public class FragmentQueryableImpl implements FragmentQueryable {

    private final Executor m_executor;

    public class ConfigurationNodeListener {

        private ConfigurationNode m_configurationNode;
        private FragmentQueryable.Listener m_listener;

        public ConfigurationNodeListener(ConfigurationNode m_configurationNode, Listener m_listener) {
            this.m_configurationNode = m_configurationNode;
            this.m_listener = m_listener;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final ConfigurationNodeListener other = (ConfigurationNodeListener) obj;
            if (this.m_configurationNode != other.m_configurationNode && (this.m_configurationNode == null || !this.m_configurationNode.equals(other.m_configurationNode))) {
                return false;
            }
            if (this.m_listener != other.m_listener && (this.m_listener == null || !this.m_listener.equals(other.m_listener))) {
                return false;
            }
            return true;
        }

        @Override
        public int hashCode() {
            int hash = 7;
            hash = 11 * hash + (this.m_configurationNode != null ? this.m_configurationNode.hashCode() : 0);
            hash = 11 * hash + (this.m_listener != null ? this.m_listener.hashCode() : 0);
            return hash;
        }

    }

    private Logger m_logger;
    private final Object m_lock = new Object();
    private Map<ConfigurationNodeListener, FragmentQueryableAgent> m_agentMap = new HashMap<ConfigurationNodeListener, FragmentQueryableAgent>();

    public FragmentQueryableImpl(Logger logger, Executor executor) {
        m_logger = logger;
        m_executor=executor;
    }

    public void subscribeFragmentsWithConfiguration(ConfigurationNode configurationNode, FragmentQueryable.Listener listener) {
        ConfigurationNodeListener cnl = new ConfigurationNodeListener(configurationNode, listener);
        try {
            synchronized (m_lock) {
                if (!m_agentMap.containsKey(cnl)) {
                    /* Create an agent and insert it into the map. */
                    FragmentQueryableAgent agent = (FragmentQueryableAgent) LimeServer.getServer().loadAgent("edu.wustl.cse.mobilab.openworkflow.comm.FragmentQueryableAgent", new java.io.Serializable[0]);
                    agent.subscribe(listener, configurationNode, m_executor);
                    m_agentMap.put(cnl, agent);
                }
            }
        } catch (LimeException le) {
            m_logger.error("Trouble loading FragmentQueryableAgent agent");
            m_logger.error(le.getMessage());
        }
    }

    public void unsubscribeFragmentsWithConfiguration(ConfigurationNode configurationNode, FragmentQueryable.Listener listener) {
        ConfigurationNodeListener cnl = new ConfigurationNodeListener(configurationNode, listener);
        synchronized (m_lock) {
            if (m_agentMap.containsKey(cnl)) {
                FragmentQueryableAgent agent = m_agentMap.remove(cnl);
                agent.unsubscribe();
            }
        }
    }
}
