package edu.wustl.cse.mobilab.openworkflow.schedule;

import edu.wustl.cse.mobilab.openworkflow.schedule.ScheduleManager.Commitment;
import edu.wustl.cse.mobilab.openworkflow.verify.Require;

public class CommitmentImpl implements Commitment {

    private final TimePeriod m_timePeriod;
    private final Location m_location;

    //------------------------------------------------------------
    public CommitmentImpl(TimePeriod timePeriod, Location location) {
        Require.neqNull(timePeriod, "timePeriod");
        Require.neqNull(location, "location");
        m_timePeriod = timePeriod;
        m_location = location;
    }

    //------------------------------------------------------------
    public int compareTo(Commitment rightCommitment) {
        return m_timePeriod.getStartTime().compareTo(rightCommitment.getTimePeriod().getStartTime());
    }

    //------------------------------------------------------------
    public TimePeriod getTimePeriod() {
        return m_timePeriod;
    }

    //------------------------------------------------------------
    public Location getLocation() {
        return m_location;
    }

    //------------------------------------------------------------
    @Override
    public boolean equals(Object that) {
        if (this == that) {
            return true;
        }
        if (!(that instanceof CommitmentImpl)) {
            return false;
        }
        CommitmentImpl commitment = (CommitmentImpl) that;
        if (!m_location.equals(commitment.m_location)) {
            return false;
        }
        if (!m_timePeriod.equals(commitment.m_timePeriod)) {
            return false;
        }
        return true;
    }

    //------------------------------------------------------------
    @Override
    public int hashCode() {
        int result = m_timePeriod.hashCode();
        result = 31 * result + m_location.hashCode();
        return result;
    }
}