package edu.wustl.cse.mobilab.openworkflow.schedule;

import edu.wustl.cse.mobilab.openworkflow.shell.Disposable;

//--------------------------------------------------------------------
/** (description of class)
 * @author Created by Louis Thomas on Nov 6, 2008. */
public interface ScheduleManager extends Disposable {
    boolean addToSchedule(Commitment commitment);
    boolean replace(Commitment oldCommitment, Commitment newCommitment);
    ServiceCommitment addToSchedule(ServiceInvocationDescriptor serviceInvocationDescriptor);
    void removeFromSchedule(Commitment commitment);
    Location getLocation(Time time);
    Location getNextLocation(Time time);
    Time getNextCommitmentTimeAfter(Time time);
    Duration getTravelTime(Location startLocation, Location endLocation);
    Time getEndOfLoiteringWindow(Time time, Location location);
    Time getEndOfLoiteringWindow(Time time);

    //################################################################

    void subscribeCommitments(CommitmentListener commitmentListener);
    void unsubscribeCommitments(CommitmentListener commitmentListener);

    interface CommitmentListener {
        void updateCommitment(Commitment oldCommitment, Commitment newCommitment);
    }

    interface Commitment extends Comparable<Commitment> {
        TimePeriod getTimePeriod();
        Location getLocation();
    }

    interface ServiceCommitment extends Commitment {
        ServiceInvocationDescriptor getServiceInvocationDescriptor();
    }

    //################################################################

    void subscribeCurrentLocation(CurrentLocationListener currentLocationListener);
    void unsubscribeCurrentLocation(CurrentLocationListener currentLocationListener);

    interface CurrentLocationListener {
        void updateCurrentLocation(Location location);
    }
}
