package edu.wustl.cse.mobilab.openworkflow.schedule;

import edu.wustl.cse.mobilab.openworkflow.verify.Require;

//--------------------------------------------------------------------
/** Default implementation of Time
 * @author Created by Louis Thomas on Feb 17, 2009. */
public class TimeImpl implements Time {

    private final long m_nTime;

    //----------------------------------------------------------------
    public TimeImpl(long nTime) {
        m_nTime=nTime;
    }

    //----------------------------------------------------------------
    public long getValue() {
        return m_nTime;
    }
    //----------------------------------------------------------------
    public TimePeriod periodToTime(Time endTime) {
        Require.neqNull(endTime, "endTime");
        return new TimePeriodImpl(this, endTime);
    }

    //----------------------------------------------------------------
    public TimePeriod periodOfDuration(Duration duration) {
        Require.neqNull(duration, "duration");
        return periodToTime(plus(duration));
    }

    //----------------------------------------------------------------
    public Time plus(Duration duration) {
        return new TimeImpl(m_nTime+duration.getValue());
    }

    //----------------------------------------------------------------
    public Time minus(Duration duration) {
        return new TimeImpl(m_nTime-duration.getValue());
    }

    //----------------------------------------------------------------
    public Duration minus(Time earlierTime) {
        return new DurationImpl(m_nTime-earlierTime.getValue());
    }

    //----------------------------------------------------------------
    public int compareTo(Time rightTime) {
        Require.neqNull(rightTime, "rightTime");
        if (m_nTime<rightTime.getValue()) {
            return -1;
        } else if (m_nTime>rightTime.getValue()) {
            return 1;
        }
        return 0;
    }

    //----------------------------------------------------------------
    @Override
    public boolean equals(Object that) {
        if (this==that) { return true; }
        if (that==null || getClass()!=that.getClass()) { return false; }
        TimeImpl time=(TimeImpl)that;
        if (m_nTime!=time.m_nTime) { return false; }
        return true;
    }

    //----------------------------------------------------------------
    @Override
    public int hashCode() {
        return (int)(m_nTime^(m_nTime>>>32));
    }

    //----------------------------------------------------------------
    @Override
    public String toString() {
        return "{Time "+m_nTime+"}";
    }
}
