package edu.wustl.cse.mobilab.openworkflow.schedule;

import edu.wustl.cse.mobilab.openworkflow.verify.Require;

//--------------------------------------------------------------------
/** Default implementation of TimePeriod
 * @author Created by Louis Thomas on Feb 17, 2009. */
public class TimePeriodImpl implements TimePeriod {

    private final Time m_startTime;
    private final Time m_endTime;

    //----------------------------------------------------------------
    public TimePeriodImpl(Time startTime, Time endTime) {
        Require.neqNull(startTime, "startTime");
        Require.neqNull(endTime, "endTime");
        Require.leqZero(startTime.compareTo(endTime), "startTime.compareTo(endTime)");
        m_startTime=startTime;
        m_endTime=endTime;
    }

    //----------------------------------------------------------------
    public Time getStartTime() {
        return m_startTime;
    }

    //----------------------------------------------------------------
    public Time getEndTime() {
        return m_endTime;
    }

    //----------------------------------------------------------------
    public Duration getDuration() {
        return m_endTime.minus(m_startTime);
    }

    //----------------------------------------------------------------
    @Override
    public boolean equals(Object that) {
        if (this==that) { return true; }
        if (that==null || getClass()!=that.getClass()) { return false; }
        TimePeriodImpl timePeriod=(TimePeriodImpl)that;
        if (!m_endTime.equals(timePeriod.m_endTime)) { return false; }
        if (!m_startTime.equals(timePeriod.m_startTime)) { return false; }
        return true;
    }

    //----------------------------------------------------------------
    @Override
    public int hashCode() {
        int result=m_startTime.hashCode();
        result=31*result+m_endTime.hashCode();
        return result;
    }

    //----------------------------------------------------------------
    @Override
    public String toString() {
        return "{TimePeriod "+m_startTime+" to "+m_endTime+"}";
    }
}
