package edu.wustl.cse.mobilab.openworkflow.service;

import edu.wustl.cse.mobilab.openworkflow.shell.ui.TabManager;
import edu.wustl.cse.mobilab.openworkflow.shell.ui.TabModel;
import edu.wustl.cse.mobilab.openworkflow.util.Nothing;
import edu.wustl.cse.mobilab.openworkflow.util.ReplyCatcher;
import edu.wustl.cse.mobilab.openworkflow.util.ReplyStatus;
import edu.wustl.cse.mobilab.openworkflow.verify.Assert;
import edu.wustl.cse.mobilab.openworkflow.verify.Require;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

//--------------------------------------------------------------------
/** (description of class)
 * @author Created by Louis Thomas on Mar 5, 2009. */
public class GenericUiService implements SimpleService {

    private static final String MESSAGE_ELEMENT="Message";
    private static final String BUTTON_ELEMENT="Button";
    private static final String OUTPUT_ELEMENT="Output";
    private static final String INPUT_ELEMENT="Input";
    private static final String REPLACE_ATTRIBUTE="Replaces";
    private static final String TEXT_AREA_ELEMENT="TextArea";
    private static final String TEXT_ATTRIBUTE="Text";
    private static final String NAME_ATTRIBUTE="Name";
    private static final String VALUE_ATTRIBUTE="Value";

    //----------------------------------------------------------------
    public Map<String, Object> invoke(Map<String, Object> parameters) {
        Assert.eqTrue(parameters.containsKey(ServiceManager.Parameters.CONFIGURATION_DATA), "parameters.containsKey(ServiceManager.Parameters.CONFIGURATION_DATA)");
        Assert.eqTrue(parameters.containsKey(ServiceManager.Parameters.TAB_MANAGER), "parameters.containsKey(ServiceManager.Parameters.TAB_MANAGER)");

        if (parameters.containsKey(ServiceManager.Parameters.LOGGER)) {
            Logger logger=(Logger)parameters.get(ServiceManager.Parameters.LOGGER);
            logger.debug("GenericUiService: Invoked with argument keys "+parameters.keySet()+".");
        }

        // create the tab
        ReplyCatcher<Nothing> replyCatcher=new ReplyCatcher<Nothing>();
        final GenericUiServiceTab genericUiServiceTab=new GenericUiServiceTab((Element)parameters.get(ServiceManager.Parameters.CONFIGURATION_DATA), parameters, replyCatcher);

        // add it to the UI
        final TabManager tabManager=(TabManager)parameters.get(ServiceManager.Parameters.TAB_MANAGER);
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                tabManager.addTab(genericUiServiceTab, TabManager.Category.DYNAMIC);
            }
        });

        // wait for user to click
        replyCatcher.waitForReply();

        // remove tab from the ui
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                tabManager.removeTab(genericUiServiceTab);
            }
        });

        // return result
        return genericUiServiceTab.getOutputParametersMap();
    }

    //----------------------------------------------------------------
    private static class GenericUiServiceTab implements TabModel {

        private final ReplyCatcher<Nothing> m_replyCatcher;
        private final Element m_formElement;
        private final Map<String, Object> m_parameters;
        private final Map<String, Object> m_outputParametersMap=new LinkedHashMap<String, Object>();
        private JPanel m_panel;

        //----------------------------------------------------------------
        public GenericUiServiceTab(Element formElement, Map<String, Object> parameters, ReplyCatcher<Nothing> replyCatcher) {
            Require.neqNull(formElement, "formElement");
            Require.neqNull(parameters, "parameters");
            Require.neqNull(replyCatcher, "replyCatcher");
            m_formElement=formElement;
            m_parameters=parameters;
            m_replyCatcher=replyCatcher;
        }

        //----------------------------------------------------------------
        // from TabModel
        public String getName() {
            Assert.isAwtThread();
            return m_formElement.getAttributeValue("Title");
        }

        //----------------------------------------------------------------
        // from TabModel
        public Component getComponent() {
            Assert.isAwtThread();

            if (null==m_panel) {
                JPanel panel=new JPanel();
                panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
                panel.setBackground(Color.white);
                panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

                Map<String, String> replaceTargetToReplacementMap=new LinkedHashMap<String, String>();

                //noinspection unchecked
                for (Element childElement : (List<Element>)m_formElement.getChildren()) {
                    if (OUTPUT_ELEMENT.equals(childElement.getName())) {
                        Assert.neqNull(childElement.getAttribute(NAME_ATTRIBUTE), "childElement.getAttribute(NAME_ATTRIBUTE)");
                        Assert.neqNull(childElement.getAttribute(VALUE_ATTRIBUTE), "childElement.getAttribute(VALUE_ATTRIBUTE)");
                        String sParameterName=childElement.getAttributeValue(NAME_ATTRIBUTE);
                        String sParameterValue=childElement.getAttributeValue(VALUE_ATTRIBUTE);
                        m_outputParametersMap.put(sParameterName, doRepacements(sParameterValue, replaceTargetToReplacementMap));

                    } else if (INPUT_ELEMENT.equals(childElement.getName())) {
                        Assert.neqNull(childElement.getAttribute(NAME_ATTRIBUTE), "childElement.getAttribute(NAME_ATTRIBUTE)");
                        Assert.neqNull(childElement.getAttribute(REPLACE_ATTRIBUTE), "childElement.getAttribute(REPLACE_ATTRIBUTE)");
                        String sParameterName=childElement.getAttributeValue(NAME_ATTRIBUTE);
                        String sParameterReplaceTarget=childElement.getAttributeValue(REPLACE_ATTRIBUTE);
                        Object replacement=m_parameters.get(sParameterName);
                        if (null!=replacement) {
                            replaceTargetToReplacementMap.put(sParameterReplaceTarget, replacement.toString());
                        }

                    } else if (MESSAGE_ELEMENT.equals(childElement.getName())) {
                        Assert.neqNull(childElement.getAttribute(TEXT_ATTRIBUTE), "childElement.getAttribute(TEXT_ATTRIBUTE)");
                        //JLabel message=new JLabel(doRepacements(childElement.getAttributeValue(TEXT_ATTRIBUTE), replaceTargetToReplacementMap));

                        JTextArea textArea=new JTextArea(doRepacements(childElement.getAttributeValue(TEXT_ATTRIBUTE), replaceTargetToReplacementMap));
                        textArea.setWrapStyleWord(true);
                        textArea.setLineWrap(true);
                        textArea.setEditable(false);


                        panel.add(textArea);
                        panel.add(Box.createVerticalStrut(4));

                    } else if (BUTTON_ELEMENT.equals(childElement.getName())) {
                        Assert.neqNull(childElement.getAttribute(TEXT_ATTRIBUTE), "childElement.getAttribute(TEXT_ATTRIBUTE)");
                        Assert.neqNull(childElement.getAttribute(NAME_ATTRIBUTE), "childElement.getAttribute(NAME_ATTRIBUTE)");
                        Assert.neqNull(childElement.getAttribute(VALUE_ATTRIBUTE), "childElement.getAttribute(VALUE_ATTRIBUTE)");
                        JButton button=new JButton(doRepacements(childElement.getAttributeValue(TEXT_ATTRIBUTE), replaceTargetToReplacementMap));
                        final String sParameterName=childElement.getAttributeValue(NAME_ATTRIBUTE);
                        final String sParameterValue=doRepacements(childElement.getAttributeValue(VALUE_ATTRIBUTE), replaceTargetToReplacementMap);
                        panel.add(button);
                        button.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                m_outputParametersMap.put(sParameterName, sParameterValue);
                                m_replyCatcher.replyNotification(ReplyStatus.SUCCEEDED, null, null);
                            }
                        });

                    } else if (TEXT_AREA_ELEMENT.equals(childElement.getName())) {
                        Assert.neqNull(childElement.getAttribute(NAME_ATTRIBUTE), "childElement.getAttribute(NAME_ATTRIBUTE)");
                        Assert.neqNull(childElement.getAttribute(VALUE_ATTRIBUTE), "childElement.getAttribute(VALUE_ATTRIBUTE)");
                        final String sParameterName=childElement.getAttributeValue(NAME_ATTRIBUTE);
                        String sParameterValue=doRepacements(childElement.getAttributeValue(VALUE_ATTRIBUTE), replaceTargetToReplacementMap);
                        m_outputParametersMap.put(sParameterName, sParameterValue);
                        final JTextArea textArea=new JTextArea(sParameterValue);
                        textArea.setRows(5);
                        textArea.setWrapStyleWord(true);
                        textArea.setLineWrap(true);
                        textArea.getDocument().addDocumentListener(new DocumentListener() {
                            public void insertUpdate(DocumentEvent e) { handleUpdate(); }
                            public void removeUpdate(DocumentEvent e) { handleUpdate(); }
                            public void changedUpdate(DocumentEvent e) { handleUpdate(); }
                            private void handleUpdate() { m_outputParametersMap.put(sParameterName, textArea.getText()); }
                        });
                        JScrollPane scrollPane=new JScrollPane(textArea);
                        panel.add(scrollPane);
                        panel.add(Box.createVerticalStrut(4));

                    } else {
                        Assert.valueNeverOccurs(childElement.getName(), "childElement.getName()");
                    }
                }

                m_panel=new JPanel(new BorderLayout());
                m_panel.add(panel, BorderLayout.NORTH);
                m_panel.setBackground(Color.white);
                m_panel.setBorder(BorderFactory.createLoweredBevelBorder());
            }
            return m_panel;
        }

        //------------------------------------------------------------
        private String doRepacements(String sMessage, Map<String, String> replaceTargetToReplacementMap) {
            for (Map.Entry<String, String> entry : replaceTargetToReplacementMap.entrySet()) {
                sMessage=sMessage.replace(entry.getKey(), entry.getValue());
            }
            return sMessage;
        }

        //------------------------------------------------------------
        public Map<String, Object> getOutputParametersMap() {
            return m_outputParametersMap;
        }
    }
}
