package edu.wustl.cse.mobilab.openworkflow.service;

import edu.wustl.cse.mobilab.openworkflow.comm.TaskFeasibilityQueryable;
import edu.wustl.cse.mobilab.openworkflow.schedule.TimePeriod;
import edu.wustl.cse.mobilab.openworkflow.shell.Disposable;
import edu.wustl.cse.mobilab.openworkflow.workflow.Task;
import edu.wustl.cse.mobilab.openworkflow.workflow.TaskNode;

import java.util.Map;

//--------------------------------------------------------------------
/** (description of class)
 * @author Created by Louis Thomas on Nov 6, 2008. */
public interface ServiceManager extends Disposable, TaskFeasibilityQueryable {

    String SERVICESTORE_NAME="servicestore";

    interface Parameters {
        String LOGGER="SeriviceManager.Parameters.LOGGER";
        String CONFIGURATION_DATA="SeriviceManager.Parameters.CONFIGURATION_DATA";
        String TAB_MANAGER="SeriviceManager.Parameters.TAB_MANAGER";
    }

    ServiceDescriptor getImplementingService(TaskNode tasknode);

    TimePeriod getTimeEstimate(Task task);  //added to accomodate bidding process

    /** Returns some metric of how specialized this host is to run
     * this task. The value v returned must be in the range
     * 0 < v <= 1. Returning 1 indicates that this host can do nothing
     * but the service associated with this task. Returning values
     * very close to 0 indicates that the service associated with the
     * task is not a specialty of this host. */
    double getCapabilityFraction(Task task);

    Map<String, Object> invokeService(ServiceDescriptor serviceDescriptor, Map<String, Object> parameters);

    //################################################################

    void subscribeServiceDescriptors(ServiceDescriptorListener serviceDescriptorListener);
    void unsubscribeServiceDescriptors(ServiceDescriptorListener serviceDescriptorListener);

    interface ServiceDescriptorListener {
        void updateServiceDescriptor(ServiceDescriptor oldServiceDescriptor, ServiceDescriptor newServiceDescriptor);
    }


}
