package edu.wustl.cse.mobilab.openworkflow.shell;

import edu.wustl.cse.mobilab.openworkflow.verify.Require;
import org.jdom.Element;

import java.util.HashMap;
import java.util.Map;

//--------------------------------------------------------------------
/** (description of class)
 * @author Created by Louis Thomas on Nov 6, 2008. */
public class InMemoryPersistenceProvider implements PersistenceProvider {

    private final Object m_stateLock=new Object();
    private final Map<String, Element> m_nameToElementMap=new HashMap<String, Element>();

    //----------------------------------------------------------------
    public Element loadXmlByName(String sName) throws NameNotFoundException {
        Require.nonempty(sName, "sName");
        synchronized (m_stateLock) {
            if (!m_nameToElementMap.containsKey(sName)) {
                 throw new NameNotFoundException(sName);
            }
            Element rootElement=m_nameToElementMap.get(sName);
            return makeCopy(rootElement);
        }
    }

    //----------------------------------------------------------------
    public void saveXmlByName(String sName, Element rootElement) {
        Require.nonempty(sName, "sName");
        Require.neqNull(rootElement, "rootElement");
        rootElement=makeCopy(rootElement);
        synchronized (m_stateLock) {
            m_nameToElementMap.put(sName, rootElement);
        }
    }

    //----------------------------------------------------------------
    private Element makeCopy(Element element) {
        return (Element)element.clone();
    }
}
