package edu.wustl.cse.mobilab.openworkflow.shell;

import edu.wustl.cse.mobilab.openworkflow.auction.AuctionManager;
import edu.wustl.cse.mobilab.openworkflow.auction.AuctionManagerImpl;
import edu.wustl.cse.mobilab.openworkflow.auction.AuctionParticipationManager;
import edu.wustl.cse.mobilab.openworkflow.auction.AuctionParticipationManagerImpl;
import edu.wustl.cse.mobilab.openworkflow.auction.TaskBidComparatorImpl;
import edu.wustl.cse.mobilab.openworkflow.comm.CommProxy;
import edu.wustl.cse.mobilab.openworkflow.comm.HostId;
import edu.wustl.cse.mobilab.openworkflow.comm.SingleProcessCommProxy;
import edu.wustl.cse.mobilab.openworkflow.comm.StringHostId;
import edu.wustl.cse.mobilab.openworkflow.schedule.LocationImpl;
import edu.wustl.cse.mobilab.openworkflow.schedule.LocationManager;
import edu.wustl.cse.mobilab.openworkflow.schedule.LocationManagerImpl;
import edu.wustl.cse.mobilab.openworkflow.schedule.ScheduleManager;
import edu.wustl.cse.mobilab.openworkflow.schedule.ScheduleManagerImpl;
import edu.wustl.cse.mobilab.openworkflow.schedule.TimeImpl;
import edu.wustl.cse.mobilab.openworkflow.service.ExecutionManager;
import edu.wustl.cse.mobilab.openworkflow.service.ExecutionManagerImpl;
import edu.wustl.cse.mobilab.openworkflow.service.GenericUiService;
import edu.wustl.cse.mobilab.openworkflow.service.ServiceManager;
import edu.wustl.cse.mobilab.openworkflow.service.ServiceManagerImpl;
import edu.wustl.cse.mobilab.openworkflow.service.SimpleService;
import edu.wustl.cse.mobilab.openworkflow.service.SimpleServiceImpl;
import edu.wustl.cse.mobilab.openworkflow.service.ui.ServicesTab;
import edu.wustl.cse.mobilab.openworkflow.shell.ui.SimpleUiMain;
import edu.wustl.cse.mobilab.openworkflow.shell.ui.TabManager;
import edu.wustl.cse.mobilab.openworkflow.shell.ui.TabManagerImpl;
import edu.wustl.cse.mobilab.openworkflow.util.ArgsHelper;
import edu.wustl.cse.mobilab.openworkflow.util.CollectionUtil;
import edu.wustl.cse.mobilab.openworkflow.util.FatalUncaughtExceptionHandler;
import edu.wustl.cse.mobilab.openworkflow.util.Nothing;
import edu.wustl.cse.mobilab.openworkflow.util.ReplyCatcher;
import edu.wustl.cse.mobilab.openworkflow.util.ReplyListener;
import edu.wustl.cse.mobilab.openworkflow.util.ReplyStatus;
import edu.wustl.cse.mobilab.openworkflow.util.SimulationTimeNotifier;
import edu.wustl.cse.mobilab.openworkflow.util.SingleThreadQueuedExecutor;
import edu.wustl.cse.mobilab.openworkflow.util.TimeNotifier;
import edu.wustl.cse.mobilab.openworkflow.util.XmlUtil;
import edu.wustl.cse.mobilab.openworkflow.verify.Assert;
import edu.wustl.cse.mobilab.openworkflow.workflow.AlphaFunction;
import edu.wustl.cse.mobilab.openworkflow.workflow.BetaFunction;
import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNode;
import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNodeImpl;
import edu.wustl.cse.mobilab.openworkflow.workflow.FragmentManager;
import edu.wustl.cse.mobilab.openworkflow.workflow.FragmentManagerImpl;
import edu.wustl.cse.mobilab.openworkflow.workflow.SetAlphaFunction;
import edu.wustl.cse.mobilab.openworkflow.workflow.SetBetaFunction;
import edu.wustl.cse.mobilab.openworkflow.workflow.WorkflowManager;
import edu.wustl.cse.mobilab.openworkflow.workflow.WorkflowManagerImpl;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Set;
import java.util.concurrent.Executor;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

//--------------------------------------------------------------------
/** A simple application demonstrating all the features of an open
 * workflow management system.
 * @author Created by Louis Thomas on Nov 6, 2008. */
public class SimpleApp {

    private Class<? extends SimpleService> m_serviceClass;

    private enum Role { CHAIR, LIB, PROF }
    private Role m_role;
    private boolean m_bShowUi;
    private boolean m_bUseLime;

    //----------------------------------------------------------------
    public static void main(String[] args) {
        SimpleApp simpleApp=new SimpleApp(args);
        simpleApp.run();
    }

    //----------------------------------------------------------------
    public SimpleApp(String[] args) {
        ArgsHelper argsHelper=new ArgsHelper(args);

        try {
            while (argsHelper.moreArgsAvailable()) {
                String sSwitch=argsHelper.getNextSwitch();
                if ("?".equalsIgnoreCase(sSwitch)) {
                    printHelp();
                    System.exit(0);
                } else if ("role".equalsIgnoreCase(sSwitch)) {
                    m_role=Role.valueOf(argsHelper.getNextArg().toUpperCase());
                } else if ("lime".equalsIgnoreCase(sSwitch)) {
                    m_bUseLime=true;
                } else if ("showui".equalsIgnoreCase(sSwitch)) {
                    m_bShowUi=true;
                } else {
                    throw new IllegalArgumentException("Unexpected switch \""+sSwitch+"\".");
                }
            }

        } catch (IllegalArgumentException e) {
            System.err.println(e.getMessage());
            printHelp();
            System.exit(1);
        }

        if (m_bShowUi) {
            m_serviceClass=GenericUiService.class;

            try {
                UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
                //UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            } catch (ClassNotFoundException e) {
                Assert.exceptionNeverCaught(e);
            } catch (InstantiationException e) {
                Assert.exceptionNeverCaught(e);
            } catch (IllegalAccessException e) {
                Assert.exceptionNeverCaught(e);
            } catch (UnsupportedLookAndFeelException e) {
                Assert.exceptionNeverCaught(e);
            }

        } else {
            m_serviceClass=SimpleServiceImpl.class;
        }
    }

    //----------------------------------------------------------------
    private static void printHelp() {
        System.out.println("Usage: "+SimpleApp.class.getName()+" <options>");
                          //----------------------------------------------------------------
        System.out.println("Options:");
        System.out.println(" -role chair|lib|prof - Which role to take. If not specified, all roles are run in process.");
        System.out.println(" -lime - Use Lime rather than in-process comm.");
        System.out.println(" -showUi - Show a user interface and require user interaction.");
    }

    //----------------------------------------------------------------
    private void run() {
        if (null==m_role) {
            runAllInProcess();
        } else {
            runOneRole(m_role);
        }
    }

    //----------------------------------------------------------------
    private void runOneRole(Role role) {
        FatalUncaughtExceptionHandler.register();

        BasicConfigurator.configure(new ConsoleAppender(new PatternLayout("%d{yyyy-MM-dd HH:mm:ss.SSS} (%-20c) %-5p - %m%n")));

        Executor executor=new SingleThreadQueuedExecutor("SimpleApp");

        SimulationTimeNotifier simulationTimeNotifier=new SimulationTimeNotifier(new TimeImpl(0));

        SingleProcessCommProxy singleProcessCommProxy=new SingleProcessCommProxy(executor, m_bUseLime);

        if (Role.CHAIR==role) {
            createDepartmentChairHost(executor, simulationTimeNotifier, singleProcessCommProxy);
        }
        if (Role.LIB==role) {
            createDepartmentLibrarianHost(executor, simulationTimeNotifier, singleProcessCommProxy);
        }
        if (Role.PROF==role) {
            final SimpleHost professorHost=createProfessorHost(executor, simulationTimeNotifier, singleProcessCommProxy);

            sleep();
            executor.execute(new Runnable() {
                public void run() {
                    ConfigurationNode initialNode=new ConfigurationNodeImpl("Idea For Class");
                    ConfigurationNode finalNode=new ConfigurationNodeImpl("Class Registered");
                    AlphaFunction alphaFunction=new SetAlphaFunction(CollectionUtil.setFromArray(initialNode));
                    BetaFunction betaFunction=new SetBetaFunction(CollectionUtil.setFromArray(finalNode));
                    Set<ConfigurationNode> initialConfigurationNodes=Collections.singleton(initialNode);
                    professorHost.getWorkflowManager().addProblem(alphaFunction, betaFunction, initialConfigurationNodes, "workflowID");
                }
            });
        }
    }

    //----------------------------------------------------------------
    private void runAllInProcess() {
        FatalUncaughtExceptionHandler.register();

        BasicConfigurator.configure(new ConsoleAppender(new PatternLayout("%d{yyyy-MM-dd HH:mm:ss.SSS} (%-20c) %-5p - %m%n")));
        final Logger logger=Logger.getLogger("System");

        Executor executor=new SingleThreadQueuedExecutor("SimpleApp");

        final SimulationTimeNotifier simulationTimeNotifier=new SimulationTimeNotifier(new TimeImpl(0));

        SingleProcessCommProxy singleProcessCommProxy=new SingleProcessCommProxy(executor, m_bUseLime);

        SimpleHost departmentChairHost=createDepartmentChairHost(executor, simulationTimeNotifier, singleProcessCommProxy);

        final SimpleHost professorHost=createProfessorHost(executor, simulationTimeNotifier, singleProcessCommProxy);

        SimpleHost departmentLibrarianHost=createDepartmentLibrarianHost(executor, simulationTimeNotifier, singleProcessCommProxy);

        // just prevent warnings
        departmentChairHost.getWorkflowManager();
        departmentLibrarianHost.getWorkflowManager();

        if (m_bShowUi) {
            LinkedHashMap<String, Object> inputParametersMap=new LinkedHashMap<String, Object>();
            inputParametersMap.put(ServiceManager.Parameters.TAB_MANAGER, professorHost.getTabManager());
            inputParametersMap.put(ServiceManager.Parameters.CONFIGURATION_DATA, XmlUtil.xmlStringToElement(
                "<Form Title=\"Begin\">"
                    +"<Message Text=\"Click OK to begin the demo and add the problem.\"/>"
                    +"<Button Text=\"OK\" Name=\"OK\" Value=\"OK\"/>"
                +"</Form>"
            ));
            new GenericUiService().invoke(inputParametersMap);
        }
        executor.execute(new Runnable() {
            public void run() {
                ConfigurationNode initialNode=new ConfigurationNodeImpl("Idea For Class");
                ConfigurationNode finalNode=new ConfigurationNodeImpl("Class Registered");
                AlphaFunction alphaFunction=new SetAlphaFunction(CollectionUtil.setFromArray(initialNode));
                BetaFunction betaFunction=new SetBetaFunction(CollectionUtil.setFromArray(finalNode));
                Set<ConfigurationNode> initialConfigurationNodes=Collections.singleton(initialNode);
                professorHost.getWorkflowManager().addProblem(alphaFunction, betaFunction, initialConfigurationNodes, "workflowID");
            }
        });

        sleep();
        executor.execute(new Runnable() {
            public void run() {
                logger.debug("-- advancing system time to 10,000 --");
                simulationTimeNotifier.setSystemTime(new TimeImpl(10000));
            }
        });

        sleep();
        executor.execute(new Runnable() {
            public void run() {
                logger.debug("-- advancing system time to 30,000 --");
                simulationTimeNotifier.setSystemTime(new TimeImpl(30000));
            }
        });

        sleep();
        executor.execute(new Runnable() {
            public void run() {
                logger.debug("-- advancing system time to 50,000 --");
                simulationTimeNotifier.setSystemTime(new TimeImpl(50000));
            }
        });

        sleep();

        if (m_bShowUi) {
            LinkedHashMap<String, Object> inputParametersMap=new LinkedHashMap<String, Object>();
            inputParametersMap.put(ServiceManager.Parameters.TAB_MANAGER, professorHost.getTabManager());
            inputParametersMap.put(ServiceManager.Parameters.CONFIGURATION_DATA, XmlUtil.xmlStringToElement(
                "<Form Title=\"Exit\">"
                    +"<Message Text=\"Click OK to exit demo.\"/>"
                    +"<Button Text=\"OK\" Name=\"OK\" Value=\"OK\"/>"
                    +"</Form>"
            ));
            new GenericUiService().invoke(inputParametersMap);
        }

        singleProcessCommProxy.dispose();

       System.exit(0);
    }

    //----------------------------------------------------------------
    private SimpleHost createDepartmentLibrarianHost(Executor executor, SimulationTimeNotifier simulationTimeNotifier, CommProxy singleProcessCommProxy) {
        PersistenceProvider persistenceProvider=new InMemoryPersistenceProvider();
        persistenceProvider.saveXmlByName(FragmentManager.FRAGMENTSTORE_NAME, XmlUtil.xmlStringToElement(
            "<Fragments>"
                +"<CompositeWorkflow>"
                    +"<Task ContractLabel=\"Order Book\" Junction=\"+\">"
                        +"<Precondition>"
                            +"<Configuration PredicateLabel=\"Book List\" />"
                        +"</Precondition>"
                        +"<Postcondition>"
                            +"<Configuration PredicateLabel=\"Book On Reserve\" />"
                        +"</Postcondition>"
                    +"</Task>"
                +"</CompositeWorkflow>"
            +"</Fragments>"
        ));
        persistenceProvider.saveXmlByName(ServiceManager.SERVICESTORE_NAME, XmlUtil.xmlStringToElement(
            "<Services>"
                +"<Service Name=\"Register Class Service\" Contract=\"Register Class\" JavaClass=\""+m_serviceClass.getName()+"\" Title=\"Register Class\">"
                    +"<Output Name=\"Class Registered\" Value=\"The class is registered.\"/>"
                    +"<Message Text=\"Please click OK to register the class.\" />"
                    +"<Button Text=\"OK\" Name=\"OK\" Value=\"OK\"/>"
                +"</Service>"
                +"<Service Name=\"Order Book Service\" Contract=\"Order Book\" JavaClass=\""+m_serviceClass.getName()+"\" Title=\"Order Book\">"
                    +"<Output Name=\"Book On Reserve\" Value=\"The book is on reserve.\"/>"
                    +"<Message Text=\"Please click OK to order the book.\" />"
                    +"<Button Text=\"OK\" Name=\"OK\" Value=\"OK\"/>"
                +"</Service>"
            +"</Services>"
        ));

        return new SimpleHost(persistenceProvider, singleProcessCommProxy, simulationTimeNotifier, simulationTimeNotifier,
            executor, Logger.getLogger("Department Librarian"), new StringHostId("DepartmentLibrarianHost"),
            m_bShowUi, "Department Librarian");
    }

    //----------------------------------------------------------------
    private SimpleHost createProfessorHost(Executor executor, SimulationTimeNotifier simulationTimeNotifier, CommProxy singleProcessCommProxy) {
        PersistenceProvider persistenceProvider=new InMemoryPersistenceProvider();
        persistenceProvider.saveXmlByName(FragmentManager.FRAGMENTSTORE_NAME, XmlUtil.xmlStringToElement(
            "<Fragments>"
                +"<CompositeWorkflow>"
                    +"<Task ContractLabel=\"Create Class\" Junction=\"+\">"
                        +"<Precondition>"
                            +"<Configuration PredicateLabel=\"Idea For Class\" />"
                        +"</Precondition>"
                        +"<Postcondition>"
                            +"<Configuration PredicateLabel=\"Class Name\" />"
                            +"<Configuration PredicateLabel=\"Syllabus\" />"
                            +"<Configuration PredicateLabel=\"Book List\" />"
                        +"</Postcondition>"
                    +"</Task>"
                +"</CompositeWorkflow>"
            +"</Fragments>"
        ));
        persistenceProvider.saveXmlByName(ServiceManager.SERVICESTORE_NAME, XmlUtil.xmlStringToElement(
            "<Services>"
                +"<Service Name=\"Create Class Service\" Contract=\"Create Class\" JavaClass=\""+m_serviceClass.getName()+"\" Title=\"Create Class\">"
                    +"<Output Name=\"Class Name\" Value=\"The Joy Of Open Workflows\"/>"
                    +"<Output Name=\"Syllabus\" Value=\"Brilliant lectures. Interesting homework assignments. Challenging tests.\"/>"
                    +"<Output Name=\"Book List\" Value=\"The Pragmatic Programmer by Hunt and Thomas\"/>"
                    +"<Message Text=\"Class Name:\" />"
                    +"<TextArea Name=\"Class Name\" Value=\"The Joy Of Open Workflows\"/>"
                    +"<Message Text=\"Syllabus:\" />"
                    +"<TextArea Name=\"Syllabus\" Value=\"Brilliant lectures. Interesting homework assignments. Challenging tests.\"/>"
                    +"<Message Text=\"Book List:\" />"
                    +"<TextArea Name=\"Book List\" Value=\"The Pragmatic Programmer by Hunt and Thomas\"/>"
                    +"<Message Text=\"Please click OK to create the class.\" />"
                    +"<Button Text=\"OK\" Name=\"OK\" Value=\"OK\"/>"
                +"</Service>"
            +"</Services>"
        ));

        return new SimpleHost(persistenceProvider, singleProcessCommProxy, simulationTimeNotifier, simulationTimeNotifier,
            executor, Logger.getLogger("Professor"), new StringHostId("ProfessorHost"),
            m_bShowUi, "Professor");
    }

    //----------------------------------------------------------------
    private SimpleHost createDepartmentChairHost(Executor executor, SimulationTimeNotifier simulationTimeNotifier, CommProxy singleProcessCommProxy) {
        PersistenceProvider persistenceProvider=new InMemoryPersistenceProvider();
        persistenceProvider.saveXmlByName(FragmentManager.FRAGMENTSTORE_NAME, XmlUtil.xmlStringToElement(
            "<Fragments>"
                +"<CompositeWorkflow>"
                    +"<Task ContractLabel=\"Approve Class\" Junction=\"+\">"
                        +"<Precondition>"
                            +"<Configuration PredicateLabel=\"Syllabus\" />"
                        +"</Precondition>"
                        +"<Postcondition>"
                            +"<Configuration PredicateLabel=\"Class Approved\" />"
                        +"</Postcondition>"
                    +"</Task>"
                +"</CompositeWorkflow>"
                +"<CompositeWorkflow>"
                    +"<Task ContractLabel=\"Register Class\" Junction=\"*\">"
                        +"<Precondition>"
                            +"<Configuration PredicateLabel=\"Class Name\" />"
                            +"<Configuration PredicateLabel=\"Class Approved\" />"
                        +"</Precondition>"
                        +"<Postcondition>"
                            +"<Configuration PredicateLabel=\"Class Registered\" />"
                        +"</Postcondition>"
                    +"</Task>"
                +"</CompositeWorkflow>"
            +"</Fragments>"
        ));
        persistenceProvider.saveXmlByName(ServiceManager.SERVICESTORE_NAME, XmlUtil.xmlStringToElement(
            "<Services>"
                +(m_bShowUi?"":
                "<Service Name=\"Register Class Service\" Contract=\"Register Class\" JavaClass=\""+m_serviceClass.getName()+"\" Title=\"Register Class\">"
                    +"<Output Name=\"Class Registered\" Value=\"The class is registered.\"/>"
                    +"<Message Text=\"Please click OK to register the class.\" />"
                    +"<Button Text=\"OK\" Name=\"OK\" Value=\"OK\"/>"
                +"</Service>"
                )
                +"<Service Name=\"Approve Class Service\" Contract=\"Approve Class\" JavaClass=\""+m_serviceClass.getName()+"\" Title=\"Approve Class\">"
                    +"<Input Name=\"Syllabus\" Replaces=\"$SYLLABUS$\"/>"
                    +"<Output Name=\"Class Approved\" Value=\"The class is approved.\"/>"
                    +"<Message Text=\"The following syllabus has been submitted for your approval:\" />"
                    +"<Message Text=\"\" />"
                    +"<Message Text=\"$SYLLABUS$\" />"
                    +"<Message Text=\"\" />"
                    +"<Message Text=\"Please click OK to approve the class.\" />"
                    +"<Button Text=\"OK\" Name=\"OK\" Value=\"OK\"/>"
                +"</Service>"
            +"</Services>"
        ));

        return new SimpleHost(persistenceProvider, singleProcessCommProxy, simulationTimeNotifier, simulationTimeNotifier,
            executor, Logger.getLogger("Department Chair"), new StringHostId("DepartmentChairHost"),
            m_bShowUi, "Department Chair");
    }

    //----------------------------------------------------------------
    private static void sleep() {
        try {
            Thread.sleep(4000);
        } catch (InterruptedException e) {
            Assert.exceptionNeverCaught(e);
        }
    }

    //################################################################

    //----------------------------------------------------------------
    private static class SimpleHost {

        private final FragmentManager m_fragmentManager;
        private final LocationManager m_locationManager;
        private final ServiceManager m_serviceManager;
        private final ExecutionManager m_executionManager;
        private final ScheduleManager m_scheduleManager;
        private final AuctionParticipationManager m_auctionParticipationManager;
        private final AuctionManager m_auctionManager;
        private final WorkflowManager m_workflowManager;
        private TabManager m_tabManager;

        //------------------------------------------------------------
        public SimpleHost(PersistenceProvider persistenceProvider, CommProxy myCommProxy, SystemTimeProvider systemTimeProvider, TimeNotifier timeNotifier, Executor executor, Logger logger, HostId hostId, boolean bShowUi, final String sUiTitle) {
            Assert.isNotAwtThread();

            if (bShowUi) {
                final ReplyCatcher<Nothing> replyCatcher=new ReplyCatcher<Nothing>();
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        createAndShowGUI(sUiTitle, replyCatcher);
                    }
                });
                replyCatcher.waitForReply();
            }

            // execution subsystem
            m_fragmentManager=new FragmentManagerImpl(persistenceProvider, logger);
            m_locationManager=new LocationManagerImpl(persistenceProvider);
            m_serviceManager=new ServiceManagerImpl(persistenceProvider, logger, m_tabManager);
            m_scheduleManager=new ScheduleManagerImpl(m_locationManager, new LocationImpl("Anywhere"));
            m_executionManager=new ExecutionManagerImpl(m_scheduleManager, m_serviceManager, myCommProxy.getExecutionDataQueryable(), timeNotifier, executor, logger);
            m_auctionParticipationManager=new AuctionParticipationManagerImpl(m_scheduleManager, m_serviceManager, systemTimeProvider, logger, hostId);

            myCommProxy.addFragmentQueryable(m_fragmentManager);
            myCommProxy.addTaskFeasibilityQueryable(m_serviceManager);
            myCommProxy.addExecutionDataQueryable(m_executionManager);
            myCommProxy.addAuctionQueryable(m_auctionParticipationManager);

            // construction subsystem
            m_auctionManager=new AuctionManagerImpl(myCommProxy.getAuctionQueryable(), new TaskBidComparatorImpl(), systemTimeProvider, timeNotifier, logger);
            m_workflowManager=new WorkflowManagerImpl(myCommProxy.getFragmentQueryable(), myCommProxy.getTaskFeasibilityQueryable(), m_auctionManager, logger);

            if (bShowUi) {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        m_tabManager.addTab(new SimpleUiMain.SingleLabelTabModel("Add Problem", "Workflow Initiator"), TabManager.Category.DYNAMIC);
                        m_tabManager.addTab(new SimpleUiMain.SingleLabelTabModel("Schedule", "ScheduleManager"), TabManager.Category.STATIC);
                        m_tabManager.addTab(new SimpleUiMain.SingleLabelTabModel("Location", "ScheduleManager"), TabManager.Category.STATIC);
                        m_tabManager.addTab(new ServicesTab(m_serviceManager), TabManager.Category.STATIC);
                        m_tabManager.addTab(new SimpleUiMain.SingleLabelTabModel("Fragments", "FragmentManager"), TabManager.Category.STATIC);
                    }
                });
            }
        }

        //------------------------------------------------------------
        public void shutdown() {
            // shutdown - call dispose on the services that need it, starting from the top

            // construction subsystem
            m_workflowManager.dispose();
            m_auctionManager.dispose();

            // execution subsystem
            m_auctionParticipationManager.dispose();
            m_scheduleManager.dispose();
            m_locationManager.dispose();
            m_executionManager.dispose();
            m_serviceManager.dispose();
            m_fragmentManager.dispose();
        }

        //------------------------------------------------------------
        public WorkflowManager getWorkflowManager() {
            return m_workflowManager;
        }

        //------------------------------------------------------------
        public ServiceManager getServiceManager() {
            return m_serviceManager;
        }

        //------------------------------------------------------------
        public ScheduleManager getScheduleManager() {
            return m_scheduleManager;
        }

        //------------------------------------------------------------
        public TabManager getTabManager() {
            return m_tabManager;
        }

        //------------------------------------------------------------
        private void createAndShowGUI(String sTitle, ReplyListener<Nothing> replyListener) {
            Assert.isAwtThread();

            TabManagerImpl tabManager=new TabManagerImpl();

            JFrame frame=new JFrame(sTitle);
            frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            frame.setSize(480, 640);
            frame.getContentPane().add(tabManager.getComponent());
            frame.setVisible(true);

            m_tabManager=tabManager;
            replyListener.replyNotification(ReplyStatus.SUCCEEDED, null, null);
        }

    }
}
