package edu.wustl.cse.mobilab.openworkflow.shell.ui;

import edu.wustl.cse.mobilab.openworkflow.service.GenericUiService;
import edu.wustl.cse.mobilab.openworkflow.service.ServiceManager;
import edu.wustl.cse.mobilab.openworkflow.service.ServiceManagerImpl;
import edu.wustl.cse.mobilab.openworkflow.service.SimpleServiceImpl;
import edu.wustl.cse.mobilab.openworkflow.service.ui.ServicesTab;
import edu.wustl.cse.mobilab.openworkflow.shell.InMemoryPersistenceProvider;
import edu.wustl.cse.mobilab.openworkflow.shell.PersistenceProvider;
import edu.wustl.cse.mobilab.openworkflow.util.Nothing;
import edu.wustl.cse.mobilab.openworkflow.util.ReplyCatcher;
import edu.wustl.cse.mobilab.openworkflow.util.ReplyStatus;
import edu.wustl.cse.mobilab.openworkflow.util.XmlUtil;
import org.apache.log4j.Logger;

import java.awt.Component;
import java.util.LinkedHashMap;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

//--------------------------------------------------------------------
/** A simple UI to exercise the UI components during development
 * @author Created by Louis Thomas on Jan 22, 2009. */
public class SimpleUiMain {

    private TabManagerImpl m_tabManager;

    //----------------------------------------------------------------
    public static void main(String[] args) {
        SimpleUiMain simpleUiMain=new SimpleUiMain();
        simpleUiMain.run();
    }

    //----------------------------------------------------------------
    private void run() {

        final ReplyCatcher<Nothing> replyCatcher=new ReplyCatcher<Nothing>();
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI(replyCatcher);
            }
        });

        replyCatcher.waitForReply();

        GenericUiService genericUiService=new GenericUiService();
        LinkedHashMap<String, Object> inputParametersMap=new LinkedHashMap<String, Object>();
        inputParametersMap.put(ServiceManager.Parameters.TAB_MANAGER, m_tabManager);
        inputParametersMap.put(ServiceManager.Parameters.CONFIGURATION_DATA, XmlUtil.xmlStringToElement(
            "<Form Title=\"Hi\">"
                +"<Message Text=\"Hello\"/>"
                +"<Button Text=\"OK\" Name=\"X\" Value=\"Y\"/>"
            +"</Form>"
        ));
        genericUiService.invoke(inputParametersMap);
    }

    //----------------------------------------------------------------
    private void createAndShowGUI(ReplyCatcher<Nothing> replyCatcher) {

        m_tabManager=new TabManagerImpl();

        SingleLabelTabModel singleLabelTabModel= new SingleLabelTabModel("name", "Hello World");
        m_tabManager.addTab(singleLabelTabModel, TabManager.Category.DYNAMIC);
        SingleLabelTabModel singleLabelTabModel2= new SingleLabelTabModel("This is a very long name", "Hello People");
        m_tabManager.addTab(singleLabelTabModel2, TabManager.Category.STATIC);
        SingleLabelTabModel singleLabelTabModel3= new SingleLabelTabModel("nom", "Hello Everyone");
        m_tabManager.addTab(singleLabelTabModel3, TabManager.Category.STATIC);
        SingleLabelTabModel singleLabelTabModel4= new SingleLabelTabModel("this is also a very long name", "Hello Children");
        m_tabManager.addTab(singleLabelTabModel4, TabManager.Category.DYNAMIC);

        ServicesTab st = new ServicesTab(makeServiceManager());
        m_tabManager.addTab(st, TabManager.Category.STATIC);

        JFrame frame=new JFrame("Open Workflow Management System - SimpleUi");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setSize(480, 640);
        frame.getContentPane().add(m_tabManager.getComponent());
        frame.setVisible(true);

        replyCatcher.replyNotification(ReplyStatus.SUCCEEDED, null, null);
    }

    //----------------------------------------------------------------
    private ServiceManager makeServiceManager() {
        PersistenceProvider persistenceProvider=new InMemoryPersistenceProvider();
        persistenceProvider.saveXmlByName(ServiceManager.SERVICESTORE_NAME, XmlUtil.xmlStringToElement(
            "<Services>"
                +"<Service Name=\"Register Class Service\" Contract=\"Register Class\" JavaClass=\""+SimpleServiceImpl.class.getName()+"\"/>"
                +"<Service Name=\"Order Book Service\" Contract=\"Order Book\" JavaClass=\""+SimpleServiceImpl.class.getName()+"\"/>"
            +"</Services>"
        ));
        return new ServiceManagerImpl(persistenceProvider, Logger.getLogger("ServiceManager"), null);
    }

    //----------------------------------------------------------------
    public static class SingleLabelTabModel implements TabModel {

        private final String m_name;
        private final Component m_component;

        public SingleLabelTabModel(String name, String text) {
            m_name=name;
            JLabel label=new JLabel(text);
            label.setOpaque(false);


            JPanel panel=new JPanel();
            panel.setOpaque(false);
            panel.add(label);


            m_component=panel;
        }

        public String getName() {
            return m_name;
        }

        public Component getComponent() {
            return m_component;
        }
    }
}
