package edu.wustl.cse.mobilab.openworkflow.util;

//--------------------------------------------------------------------
/** Indicates an object that can receive notifications in the form of
 * standard replies, which includes the possibility that the
 * connection was lost before the request could be completely
 * processed.
 * <P> The SuccessType is the expected type of the replyDetail on
 * success. If replyDetail will be null, use {@link Nothing} as the
 * SuccessType.
 * @author Created by Louis Thomas on Apr 25, 2006 */
public interface ReplyListener<SuccessType> {

    /** A reply to a previus request has been recieved.
     * @param replyStatus the result of the request.
     * @param replyDetail a custom reply value whose value depends
     * upon the request made. On success, this object should be of
     * type SuccessType. When a request is denied, the replyDetail
     * will usually be an Exception or a String (so that toString may
     * always be called to get a displayable error message). Will be
     * null if there was a disconnect.
     * @param extra the extra parameter that was passed to the
     * request, which is often used to itentify which reply goes with
     * which request. */
    void replyNotification(ReplyStatus replyStatus, Object replyDetail, Object extra);
}
