package edu.wustl.cse.mobilab.openworkflow.workflow;

import edu.wustl.cse.mobilab.openworkflow.util.CollectionUtil;
import edu.wustl.cse.mobilab.openworkflow.verify.Require;

import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

//--------------------------------------------------------------------
/** A simple beta function that requires all of the given
 * configuration nodes be final configurations.
 * @author Created by Louis Thomas on Jan 15, 2009. */
public class SetBetaFunction implements BetaFunction {

    private Set<ConfigurationNode> m_finalConfigurationNodes;

    //----------------------------------------------------------------
    public SetBetaFunction(Set<ConfigurationNode> finalConfigurationNodes) {
        Require.neqNull(finalConfigurationNodes, "finalConfigurationNodes");
        Require.gtZero(finalConfigurationNodes.size(), "finalConfigurationNodes.size()");
        m_finalConfigurationNodes=Collections.unmodifiableSet(new LinkedHashSet<ConfigurationNode>(finalConfigurationNodes));
    }

    //----------------------------------------------------------------
    public Set<ConfigurationNode> identifyFinalConfigurations(Set<ConfigurationNode> configurationNodes) {
        Require.neqNull(configurationNodes, "configurationNodes");
        if (configurationNodes.containsAll(m_finalConfigurationNodes)) {
            return m_finalConfigurationNodes;
        }
        return null;
    }

    //----------------------------------------------------------------
    @Override
    public String toString() {
        return "{SetBetaFunction "+CollectionUtil.formatSetStringSorted(m_finalConfigurationNodes)+"}";
    }
}
