package edu.wustl.cse.mobilab.openworkflow.workflow;

import edu.wustl.cse.mobilab.openworkflow.verify.Require;

import java.io.Serializable;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

//--------------------------------------------------------------------
/** (description of class)
 * @author Created by Louis Thomas on Dec 2, 2008. */
public class TaskNodeImpl implements TaskNode, Serializable {

    private static final long serialVersionUID = 1;
    private final String m_sContractLabel;
    private final Junction m_junction;
    private final Set<ConfigurationNode> m_preconditionConfigurationNodes;
    private final Set<ConfigurationNode> m_postconditionConfigurationNodes;

    //----------------------------------------------------------------
    public TaskNodeImpl(String sContractLabel, Junction junction, Set<ConfigurationNode> preconditionConfigurationNodes, Set<ConfigurationNode> postconditionConfigurationNodes) {
        Require.nonempty(sContractLabel, "sContractLabel");
        Require.neqNull(junction, "junction");
        if (null==preconditionConfigurationNodes) {
            preconditionConfigurationNodes=Collections.emptySet();
        }
        if (null==postconditionConfigurationNodes) {
            postconditionConfigurationNodes=Collections.emptySet();
        }
        Require.eqFalse(preconditionConfigurationNodes.isEmpty(), "preconditionConfigurationNodes.isEmpty()");
        Require.eqFalse(postconditionConfigurationNodes.isEmpty(), "postconditionConfigurationNodes.isEmpty()");

        m_sContractLabel=sContractLabel;
        m_junction=junction;
        m_preconditionConfigurationNodes=Collections.unmodifiableSet(new LinkedHashSet<ConfigurationNode>(preconditionConfigurationNodes));
        m_postconditionConfigurationNodes=Collections.unmodifiableSet(new LinkedHashSet<ConfigurationNode>(postconditionConfigurationNodes));
    }

    //----------------------------------------------------------------
    @Override
    public boolean equals(Object that) {
        if (this==that) { return true; }
        if (that==null || getClass()!=that.getClass()) { return false; }
        TaskNodeImpl taskNode=(TaskNodeImpl)that;
        if (m_junction!=taskNode.m_junction) { return false; }
        if (!m_postconditionConfigurationNodes.equals(taskNode.m_postconditionConfigurationNodes)) { return false; }
        if (!m_preconditionConfigurationNodes.equals(taskNode.m_preconditionConfigurationNodes)) { return false; }
        if (!m_sContractLabel.equals(taskNode.m_sContractLabel)) { return false; }
        return true;
    }

    //----------------------------------------------------------------
    @Override
    public int hashCode() {
        int result=m_sContractLabel.hashCode();
        result=31*result+m_junction.hashCode();
        result=31*result+m_preconditionConfigurationNodes.hashCode();
        result=31*result+m_postconditionConfigurationNodes.hashCode();
        return result;
    }

    //----------------------------------------------------------------
    @Override
    public String toString() {
        return "{TaskNode contractLabel:\""+m_sContractLabel+"\" junction:"+m_junction+" precondition:"+m_preconditionConfigurationNodes+" postcondition:"+m_postconditionConfigurationNodes+"}";
    }

    //----------------------------------------------------------------
    public String getContractLabel() {
        return m_sContractLabel;
    }

    //----------------------------------------------------------------
    public Set<ConfigurationNode> getPreconditions() {
        return m_preconditionConfigurationNodes;
    }

    //----------------------------------------------------------------
    public Set<ConfigurationNode> getPostconditions() {
        return m_postconditionConfigurationNodes;
    }

    //----------------------------------------------------------------
    public Junction getJunction() {
        return m_junction;
    }
}
