package edu.wustl.cse.mobilab.openworkflow.workflow;

import edu.wustl.cse.mobilab.openworkflow.shell.Disposable;
import edu.wustl.cse.mobilab.openworkflow.util.Tuple2;

import java.util.Set;

//--------------------------------------------------------------------
/** (description of class)
 * @author Created by Louis Thomas on Nov 6, 2008. */
public interface WorkflowManager extends Disposable {

    void addProblem(AlphaFunction alphaFunction, BetaFunction betaFunction, Set<ConfigurationNode> initialConfigurationNodes, String workflowID);

    void addListener(Listener listener);
    void removeListener(Listener listener);

    //----------------------------------------------------------------
    interface Listener {
        /** Notify that a new workspace has been added.*/
        void handleProblemAdded(Workspace workspace);
    }

    //----------------------------------------------------------------
    /** A model that exposes the current state of a workflow problem. */
    interface Workspace {
        enum Color { BLUE, PURPLE, GREEN, RED, UNCOLORED } // order is significant!

        AlphaFunction getAlphaFunction();
        BetaFunction getBetaFunction();
        Set<ConfigurationNode> getInitialConfigurationNodes();
        String getWorkflowId();
        Set<TaskNode> getTaskNodes();

        Color getNodeColor(Node node);
        /** May only be called if the node is &lt;= green, */
        int getNodeDistance(Node node);
        /** May only be called if the edge is in the graph. */
        Color getEdgeColor(Tuple2<Node, Node> edge);

        void addListener(Listener listener);
        void removeListener(Listener listener);

        //------------------------------------------------------------
        interface Listener {
            void taskNodeAdded(TaskNode taskNode);
            void nodeProperyChanged(Node node);
            void edgeProperyChanged(Tuple2<Node, Node> edge);
        }
    }
}
