package edu.wustl.cse.mobilab.openworkflow.auction;

import static org.easymock.EasyMock.and;
import static org.easymock.EasyMock.anyObject;
import static org.easymock.EasyMock.capture;
import static org.easymock.EasyMock.eq;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.easymock.Capture;
import org.easymock.EasyMock;

import edu.wustl.cse.mobilab.openworkflow.comm.AuctionQueryable;
import edu.wustl.cse.mobilab.openworkflow.comm.HostId;
import edu.wustl.cse.mobilab.openworkflow.comm.StringHostId;
import edu.wustl.cse.mobilab.openworkflow.comm.AuctionQueryable.Listener;
import edu.wustl.cse.mobilab.openworkflow.schedule.ScheduleManager;
import edu.wustl.cse.mobilab.openworkflow.schedule.ServiceInvocationDescriptor;
import edu.wustl.cse.mobilab.openworkflow.schedule.Time;
import edu.wustl.cse.mobilab.openworkflow.schedule.TimeImpl;
import edu.wustl.cse.mobilab.openworkflow.service.ServiceDescriptorImpl;
import edu.wustl.cse.mobilab.openworkflow.service.ServiceManager;
import edu.wustl.cse.mobilab.openworkflow.shell.SystemTimeProvider;
import edu.wustl.cse.mobilab.openworkflow.util.BaseTestCase;
import edu.wustl.cse.mobilab.openworkflow.util.SimulationTimeNotifier;
import edu.wustl.cse.mobilab.openworkflow.util.TimeNotifier;
import edu.wustl.cse.mobilab.openworkflow.verify.RequirementFailure;
import edu.wustl.cse.mobilab.openworkflow.workflow.Task;

/**
 * Tests for AuctionManagerImpl.
 * 
 * @author sf7
 */
public class TestAuctionManagerImpl extends BaseTestCase {

    AuctionQueryable mockAuctionQueryable;
    TaskBidComparator taskBidComparator;
    SystemTimeProvider timeProvider;
    SimulationTimeNotifier timeNotifier;
    Logger logger;
    LoggingRecorder loggingRecorder;
    Set<Object> mocks;

    private void makeConstructorMocks() {
        mocks=new HashSet<Object>();
        mockAuctionQueryable = createMock(AuctionQueryable.class);
        taskBidComparator = new TaskBidComparatorImpl();
        timeNotifier= new SimulationTimeNotifier(new TimeImpl(0));
        timeProvider = timeNotifier;
        logger=Logger.getLogger(AuctionManagerImpl.class);
        loggingRecorder=new LoggingRecorder(logger);
    }

    private <T> T createMock(Class<T> c) {
        T mock=EasyMock.createMock(c);
        mocks.add(mock);
        return mock;
    }
    
    public void testSimpleConfirmBeforeDeadline() {
        // Set up mocks
        makeConstructorMocks();
        
        Task mockTask = createMock(Task.class);
        
        Capture<AuctionQueryable.Listener> listenerCapture = 
            new Capture<AuctionQueryable.Listener>();
        mockAuctionQueryable.startSolicitingBids(eq(mockTask),
                (AuctionQueryable.Listener) and(anyObject(),capture(listenerCapture)));
        
        final Time deadline = new TimeImpl(100);
        TaskBid taskBid = new TaskBidImpl(mockTask, 
                new StringHostId("Host Foo"), 0.5, deadline);
        
        AuctionQueryable.Bidder mockBidder = new AuctionQueryable.Bidder() {

            public void confirmBid(TaskBid taskBid) {
                assertTrue(timeProvider.getSystemTime().compareTo(deadline) <= 0);
            }

            public void rejectBid(TaskBid taskBid) {
                fail("Should not reject bid.");
            }  
        };
        
        mockAuctionQueryable.stopSolicitingBids(eq(mockTask),
                (AuctionQueryable.Listener)anyObject());
        
        // Do test
        replay(mocks.toArray());
        AuctionManagerImpl auctionManager = new AuctionManagerImpl(
                mockAuctionQueryable, taskBidComparator, timeProvider,
                timeNotifier, logger);
        
        auctionManager.allocateTask(mockTask);
        if (!listenerCapture.hasCaptured())
            fail("Failed to capture listener.");
        listenerCapture.getValue().handleBid(mockBidder, taskBid);
        timeNotifier.setSystemTime(deadline);
        
        // End test
        verify(mocks.toArray());
        loggingRecorder.detatch();
    }
  }
