package edu.wustl.cse.mobilab.openworkflow.schedule;

import edu.wustl.cse.mobilab.openworkflow.schedule.ScheduleManager.Commitment;
import edu.wustl.cse.mobilab.openworkflow.util.BaseTestCase;
import edu.wustl.cse.mobilab.openworkflow.util.CollectionUtil;
import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNode;
import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNodeImpl;
import edu.wustl.cse.mobilab.openworkflow.workflow.ExecutionDataName;
import edu.wustl.cse.mobilab.openworkflow.workflow.Junction;
import edu.wustl.cse.mobilab.openworkflow.workflow.ParameterName;
import edu.wustl.cse.mobilab.openworkflow.workflow.Task;
import edu.wustl.cse.mobilab.openworkflow.workflow.TaskImpl;
import edu.wustl.cse.mobilab.openworkflow.workflow.TaskNode;
import edu.wustl.cse.mobilab.openworkflow.workflow.TaskNodeImpl;

import java.util.LinkedHashMap;
import java.util.Map;


//--------------------------------------------------------------------
/** Tests for {@link ScheduleManager}
 * @author Created by Louis Thomas on Jan 15, 2009. */
public class TestScheduleManagerImpl extends BaseTestCase {

    //----------------------------------------------------------------
    public void testScheduleManager() {
        //need some other managers to make a schedule manager
        LocationManager locationManager=new MockLocationManager();

        ConfigurationNode configurationNodeA0=new ConfigurationNodeImpl("A0");
        ConfigurationNode configurationNodeA2=new ConfigurationNodeImpl("A2");
        TaskNode taskNodeA1=new TaskNodeImpl("A1", Junction.DISJUNCT, CollectionUtil.setFromArray(configurationNodeA0), CollectionUtil.setFromArray(configurationNodeA2));

        ConfigurationNode configurationNodeB0=new ConfigurationNodeImpl("B0");
        ConfigurationNode configurationNodeB2=new ConfigurationNodeImpl("B2");
        TaskNode taskNodeB1=new TaskNodeImpl("B1", Junction.DISJUNCT, CollectionUtil.setFromArray(configurationNodeB0), CollectionUtil.setFromArray(configurationNodeB2));

        //test constructor
        Location startLoc = new LocationImpl("Cave");
        ScheduleManager scheduleManager=new ScheduleManagerImpl(locationManager, startLoc);

        //test addtoSchedule -- needs functioning Task.getLocation, which requires a location implementation and a service implementation
        Time startTime = new TimeImpl(5);
        Time endTime = new TimeImpl(50);
        Time expireTime = new TimeImpl(100);
        Time endTime2 = new TimeImpl(145);
        Time testTime = new TimeImpl(25);
        Time beforeTime = new TimeImpl(0);
        Time betweenTime = new TimeImpl(75);
        TimePeriod timePeriod = new TimeImpl(5).periodOfDuration(new DurationImpl(45));
        TimePeriod secondTimePeriod = new TimeImpl(100).periodOfDuration(new DurationImpl(45));
        TimePeriod testTimePeriod= new TimeImpl(50).periodOfDuration(new DurationImpl(50));
        TimePeriod testTimePeriod2= new TimeImpl(44).periodOfDuration(new DurationImpl(54));
        TimePeriod testTimePeriod3= new TimeImpl(6).periodOfDuration(new DurationImpl(44));
        Location taskLocation = new LocationImpl("Meadow");
        Map<ExecutionDataName,ParameterName> inputMap=new LinkedHashMap();
        Map<ExecutionDataName,ParameterName> outputMap=new LinkedHashMap();
        Task task = new TaskImpl(taskNodeA1, timePeriod, taskLocation, inputMap, outputMap);
        Task task2 = new TaskImpl(taskNodeB1, secondTimePeriod, startLoc, inputMap, outputMap);
        //scheduleManager.addToSchedule(task);
        //scheduleManager.addToSchedule(task2);

        //test interval obligation tests
        //assertFalse(scheduleManager.hasCommitmentDuring(testTimePeriod));
        //assertTrue(scheduleManager.hasCommitmentDuring(testTimePeriod2));
        //assertTrue(scheduleManager.hasCommitmentDuring(testTimePeriod3));

        //test location lookups
        //assertEquals(new LocationImpl("Meadow"), scheduleManager.getLocation(testTime));
        //assertEquals(new LocationImpl("Cave"), scheduleManager.getLocation(beforeTime));
        //assertEquals(new LocationImpl("Meadow"), scheduleManager.getLocation(betweenTime));
        //assertEquals(new LocationImpl("Cave"), scheduleManager.getLocation(expireTime));
        //assertEquals(new LocationImpl("Cave"), scheduleManager.getNextLocation(betweenTime));
        //assertEquals(new LocationImpl("Meadow"), scheduleManager.getNextLocation(beforeTime));
    }

    public void testEmptyIntervalOverlap() {
        LocationManager locationManager=new LocationManager() {
            public Duration getTravelTime(Location startLocation, Location endLocation) {
                return new DurationImpl(0);
            }
            public void dispose() {
            }
        };
        Location startLoc = new LocationImpl("Cave");
        ScheduleManager scheduleManager=new ScheduleManagerImpl(locationManager, startLoc);
        Commitment commitmentA = new CommitmentImpl(new TimePeriodImpl(new TimeImpl(0), new TimeImpl(5000)), startLoc);
        Commitment commitmentB = new CommitmentImpl(new TimePeriodImpl(new TimeImpl(0), new TimeImpl(0)), startLoc);
        assertTrue(scheduleManager.addToSchedule(commitmentA));
        assertFalse(scheduleManager.addToSchedule(commitmentB));
    }

    public void testReplace() {
        LocationManager locationManager=new LocationManager() {
            public Duration getTravelTime(Location startLocation, Location endLocation) {
                return new DurationImpl(0);
            }
            public void dispose() {
            }
        };
        Location startLoc = new LocationImpl("Cave");
        ScheduleManager scheduleManager=new ScheduleManagerImpl(locationManager, startLoc);
        Commitment commitmentA = new CommitmentImpl(new TimePeriodImpl(new TimeImpl(0), new TimeImpl(500)), startLoc);
        Commitment commitmentB = new CommitmentImpl(new TimePeriodImpl(new TimeImpl(600), new TimeImpl(1000)), startLoc);
        Commitment commitmentC = new CommitmentImpl(new TimePeriodImpl(new TimeImpl(400), new TimeImpl(700)), startLoc);
        Commitment commitmentD = new CommitmentImpl(new TimePeriodImpl(new TimeImpl(700), new TimeImpl(900)), startLoc);

        assertTrue(scheduleManager.addToSchedule(commitmentA));
        assertTrue(scheduleManager.addToSchedule(commitmentB));
        assertFalse(scheduleManager.addToSchedule(commitmentC));
        assertFalse(scheduleManager.replace(commitmentA, commitmentC));
        assertFalse(scheduleManager.replace(commitmentB, commitmentC));
        assertFalse(scheduleManager.addToSchedule(commitmentD));
        assertFalse(scheduleManager.replace(commitmentA, commitmentD));
        assertTrue(scheduleManager.replace(commitmentB, commitmentD));
        assertFalse(scheduleManager.addToSchedule(commitmentB));
        assertTrue(scheduleManager.replace(commitmentD, commitmentB));
    }
}
