package edu.wustl.cse.mobilab.openworkflow.service;

import edu.wustl.cse.mobilab.openworkflow.comm.TaskFeasibilityQueryable;
import edu.wustl.cse.mobilab.openworkflow.shell.InMemoryPersistenceProvider;
import edu.wustl.cse.mobilab.openworkflow.util.BaseTestCase;
import edu.wustl.cse.mobilab.openworkflow.util.RecordingMockObject;
import edu.wustl.cse.mobilab.openworkflow.util.SimpleXmlSerializer;
import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNode;
import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNodeImpl;
import edu.wustl.cse.mobilab.openworkflow.workflow.Junction;
import edu.wustl.cse.mobilab.openworkflow.workflow.TaskNode;
import edu.wustl.cse.mobilab.openworkflow.workflow.TaskNodeImpl;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.Collections;

//--------------------------------------------------------------------
/** Tests for {@link ServiceManagerImpl}
 * @author Created by Louis Thomas on Jan 15, 2009. */
public class TestServiceManagerImpl extends BaseTestCase {

    private LoggingRecorder m_loggingRecorder;

    //----------------------------------------------------------------
    @Override
    protected void setUp() {
        super.setUp();
        m_loggingRecorder=new LoggingRecorder(Logger.getLogger(ServiceManagerImpl.class));
    }

    //----------------------------------------------------------------
    @Override
    protected void tearDown() {
        m_loggingRecorder.detatch();
        super.tearDown();
    }

    //----------------------------------------------------------------
    public void testServiceManager() {
        //create a test task node
        ConfigurationNode configurationNodeA = new ConfigurationNodeImpl("A");
        ConfigurationNode configurationNodeC = new ConfigurationNodeImpl("C");
        TaskNode taskNode = new TaskNodeImpl("B", Junction.DISJUNCT, Collections.<ConfigurationNode>singleton(configurationNodeA), Collections.<ConfigurationNode>singleton(configurationNodeC));

        //create a test service
        ServiceDescriptor serviceDescriptor=new ServiceDescriptorImpl("S1", "B", "testClass", null);

        SimpleXmlSerializer simpleXmlSerializer = new SimpleXmlSerializer();
        Element serviceNodeElement = simpleXmlSerializer.serializeServiceNode(serviceDescriptor);

        Element rootElement = new Element("Services");
        rootElement.addContent(serviceNodeElement);

        InMemoryPersistenceProvider inMemoryPersistenceProvider = new InMemoryPersistenceProvider();
        inMemoryPersistenceProvider.saveXmlByName(ServiceManager.SERVICESTORE_NAME, rootElement);

        ServiceManager serviceManager=new ServiceManagerImpl(inMemoryPersistenceProvider, Logger.getLogger(ServiceManagerImpl.class), null);

        MockServiceQueryableListener mockServiceQueryableListener = new MockServiceQueryableListener();

        // find any services that can do task "B"
        serviceManager.subscribeTaskFeasibility(taskNode, mockServiceQueryableListener);
        assertEquals("handleTaskIsFeasible()", mockServiceQueryableListener.getActivityRecordAndReset());
    }

    //----------------------------------------------------------------
    private static class MockServiceQueryableListener extends RecordingMockObject implements TaskFeasibilityQueryable.Listener {
        public void handleTaskIsFeasible() {
            recordActivity("handleTaskIsFeasible()");
        }
    }
}