package edu.wustl.cse.mobilab.openworkflow.util;

import edu.wustl.cse.mobilab.openworkflow.service.ServiceDescriptor;
import edu.wustl.cse.mobilab.openworkflow.service.ServiceDescriptorImpl;
import edu.wustl.cse.mobilab.openworkflow.verify.RequirementFailure;
import edu.wustl.cse.mobilab.openworkflow.workflow.CompositeWorkflow;
import edu.wustl.cse.mobilab.openworkflow.workflow.CompositeWorkflowBuilder;
import edu.wustl.cse.mobilab.openworkflow.workflow.CompositeWorkflowBuilderImpl;
import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNode;
import edu.wustl.cse.mobilab.openworkflow.workflow.ConfigurationNodeImpl;
import edu.wustl.cse.mobilab.openworkflow.workflow.Junction;
import edu.wustl.cse.mobilab.openworkflow.workflow.TaskNode;
import edu.wustl.cse.mobilab.openworkflow.workflow.TaskNodeImpl;
import edu.wustl.cse.mobilab.openworkflow.workflow.XmlDeserializationException;
import org.jdom.Element;

import java.io.IOException;
import java.util.Collections;
import java.util.Set;

//--------------------------------------------------------------------
/** Tests for {@link SimpleXmlSerializer}
 * @author Created by Louis Thomas on Dec 3, 2008. */
public class TestSimpleXmlSerializer extends BaseTestCase {

    SimpleXmlSerializer m_simpleXmlSerializer;

    //----------------------------------------------------------------
    public void testSerializeServiceNode() throws XmlDeserializationException, IOException {
        SimpleXmlSerializer simpleXmlSerializer=new SimpleXmlSerializer();
        ServiceDescriptor serviceDescriptor= new ServiceDescriptorImpl("S1","B","testClass", null);
        Element element = simpleXmlSerializer.serializeServiceNode(serviceDescriptor);
        assertEquals("<Service Name=\"S1\" Contract=\"B\" JavaClass=\"testClass\" />", XmlUtil.getElementAsXmlString(element));
    }

    //----------------------------------------------------------------
    public void testSerializeConfigurationNode() throws XmlDeserializationException, IOException {
        SimpleXmlSerializer simpleXmlSerializer=new SimpleXmlSerializer();
        ConfigurationNode configurationNode = new ConfigurationNodeImpl("A");
        Element element = simpleXmlSerializer.serializeConfigurationNode(configurationNode);
        assertEquals("<Configuration PredicateLabel=\"A\" />", XmlUtil.getElementAsXmlString(element));
    }

    //----------------------------------------------------------------
    public void testSerializeTaskNode() throws XmlDeserializationException, IOException {
        SimpleXmlSerializer simpleXmlSerializer=new SimpleXmlSerializer();
        ConfigurationNode configurationNodeA = new ConfigurationNodeImpl("A");
        ConfigurationNode configurationNodeC = new ConfigurationNodeImpl("C");
        TaskNode taskNode = new TaskNodeImpl("B", Junction.DISJUNCT, Collections.<ConfigurationNode>singleton(configurationNodeA), Collections.<ConfigurationNode>singleton(configurationNodeC));
        Element element = simpleXmlSerializer.serializeTaskNode(taskNode);
        assertEquals("<Task ContractLabel=\"B\" Junction=\"+\"><Precondition><Configuration PredicateLabel=\"A\" /></Precondition><Postcondition><Configuration PredicateLabel=\"C\" /></Postcondition></Task>", XmlUtil.getElementAsXmlString(element));
    }

    //----------------------------------------------------------------
    public void testSerializeCompositeWorkflow() throws XmlDeserializationException, IOException {
        SimpleXmlSerializer simpleXmlSerializer=new SimpleXmlSerializer();
        ConfigurationNode configurationNodeA = new ConfigurationNodeImpl("A");
        ConfigurationNode configurationNodeC = new ConfigurationNodeImpl("C");
        TaskNode taskNode = new TaskNodeImpl("B", Junction.DISJUNCT, Collections.<ConfigurationNode>singleton(configurationNodeA), Collections.<ConfigurationNode>singleton(configurationNodeC));
        CompositeWorkflowBuilder compositeWorkflowBuilder=new CompositeWorkflowBuilderImpl();
        compositeWorkflowBuilder.addTaskNode(taskNode);
        Element element = simpleXmlSerializer.serializeCompositeWorkflow(compositeWorkflowBuilder);
        assertEquals("<CompositeWorkflow><Task ContractLabel=\"B\" Junction=\"+\"><Precondition><Configuration PredicateLabel=\"A\" /></Precondition><Postcondition><Configuration PredicateLabel=\"C\" /></Postcondition></Task></CompositeWorkflow>", XmlUtil.getElementAsXmlString(element));
    }

    //################################################################

    //----------------------------------------------------------------
    public void testDeserializeServiceNode() throws XmlDeserializationException {
        SimpleXmlSerializer simpleXmlSerializer=new SimpleXmlSerializer();

        // null input param
        try {
            simpleXmlSerializer.deserializeServiceNode(null);
            fail("Expected null root element to fail.");
        } catch (RequirementFailure e) {
            assertTrue(e.isThisStackFrameCulprit(0));
        }

        // wrong element name
        Element wrongElement=new Element("Wrong");
        try {
            simpleXmlSerializer.deserializeServiceNode(wrongElement);
            fail("Expected bad name to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Expected \"Service\" element, but got \"Wrong\" element.", e.getMessage());
        }

        Element serviceNodeElementS1=new Element(SimpleXmlSerializer.SERVICE_NODE_ELEMENT);
        serviceNodeElementS1.setAttribute(SimpleXmlSerializer.SERVICE_NAME_ATTRIBUTE, "S1");
        serviceNodeElementS1.setAttribute(SimpleXmlSerializer.SERVICE_CONTRACT_ATTRIBUTE, "B");
        serviceNodeElementS1.setAttribute(SimpleXmlSerializer.SERVICE_JAVACLASS_ATTRIBUTE, "testClass");

        // normal
        ServiceDescriptor serviceDescriptor=simpleXmlSerializer.deserializeServiceNode(serviceNodeElementS1);
        // *************************NOT SURE HOW TO SUCCESSFULLY COMPARE THESE!
        assertEquals(new ServiceDescriptorImpl("S1", "B", "testClass", null), serviceDescriptor);

        // missing service name attribute
        serviceNodeElementS1.removeAttribute(SimpleXmlSerializer.SERVICE_NAME_ATTRIBUTE);
        try{
            simpleXmlSerializer.deserializeServiceNode(serviceNodeElementS1);
            fail("Expected missing attribute to fail.");
        } catch (XmlDeserializationException e){
            assertEquals("Missing \"Name\" attribute for \"Service\" element.", e.getMessage());
        }
        serviceNodeElementS1.setAttribute(SimpleXmlSerializer.SERVICE_NAME_ATTRIBUTE, "S1");

        // missing contract attribute
        serviceNodeElementS1.removeAttribute(SimpleXmlSerializer.SERVICE_CONTRACT_ATTRIBUTE);
        try {
            simpleXmlSerializer.deserializeServiceNode(serviceNodeElementS1);
            fail("Expected missing attribute to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Missing \"Contract\" attribute for \"Service\" element.", e.getMessage());
        }
        serviceNodeElementS1.setAttribute(SimpleXmlSerializer.SERVICE_CONTRACT_ATTRIBUTE, "B");

        // missing classname attribute
        serviceNodeElementS1.removeAttribute(SimpleXmlSerializer.SERVICE_JAVACLASS_ATTRIBUTE);
        try {
            simpleXmlSerializer.deserializeServiceNode(serviceNodeElementS1);
            fail("Expected missing attribute to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Missing \"JavaClass\" attribute for \"Service\" element.", e.getMessage());
        }
        serviceNodeElementS1.setAttribute(SimpleXmlSerializer.SERVICE_JAVACLASS_ATTRIBUTE, "testClass");
    }


    //----------------------------------------------------------------
    public void testDeserializeConfigurationNode() throws XmlDeserializationException {
        SimpleXmlSerializer simpleXmlSerializer=new SimpleXmlSerializer();

        // null input param
        try {
            simpleXmlSerializer.deserializeConfigurationNode(null);
            fail("Expected null root element to fail.");
        } catch (RequirementFailure e) {
            assertTrue(e.isThisStackFrameCulprit(0));
        }

        // wrong element name
        Element wrongElement=new Element("Wrong");
        try {
            simpleXmlSerializer.deserializeConfigurationNode(wrongElement);
            fail("Expected bad name to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Expected \"Configuration\" element, but got \"Wrong\" element.", e.getMessage());
        }

        // missing predicate label attribute
        Element configurationNodeElement=new Element(SimpleXmlSerializer.CONFIGURATION_NODE_ELEMENT);
        try {
            simpleXmlSerializer.deserializeConfigurationNode(configurationNodeElement);
            fail("Expected missing attribute to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Missing \"PredicateLabel\" attribute for \"Configuration\" element.", e.getMessage());
        }

        // valid configuration node
        configurationNodeElement.setAttribute(SimpleXmlSerializer.PREDICATE_LABEL_ATTRIBUTE, "MyPredicate");
        ConfigurationNode configurationNode=simpleXmlSerializer.deserializeConfigurationNode(configurationNodeElement);
        assertEquals(new ConfigurationNodeImpl("MyPredicate"), configurationNode);
    }

    //----------------------------------------------------------------
    public void testDeserializeConfigurationNodeSet() throws XmlDeserializationException {
        SimpleXmlSerializer simpleXmlSerializer=new SimpleXmlSerializer();

        // null input param
        try {
            simpleXmlSerializer.deserializeConfigurationNodeSet(null);
            fail("Expected null root element to fail.");
        } catch (RequirementFailure e) {
            assertTrue(e.isThisStackFrameCulprit(0));
        }

        // empty set
        Element configurationNodeSetElement=new Element("NodeSet");
        Set<ConfigurationNode> configurationNodeSet=simpleXmlSerializer.deserializeConfigurationNodeSet(configurationNodeSetElement);
        assertCollectionContainsExactly(configurationNodeSet);

        // wrong element name
        Element wrongElement=new Element("Wrong");
        configurationNodeSetElement.addContent(wrongElement);
        try {
            simpleXmlSerializer.deserializeConfigurationNodeSet(configurationNodeSetElement);
            fail("Expected bad name to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Unknown child element \"Wrong\" in \"NodeSet\" element.", e.getMessage());
        }
        configurationNodeSetElement.removeContent(wrongElement);

        // one element set
        Element configurationNodeElement=new Element(SimpleXmlSerializer.CONFIGURATION_NODE_ELEMENT);
        configurationNodeElement.setAttribute(SimpleXmlSerializer.PREDICATE_LABEL_ATTRIBUTE, "MyPredicate");
        configurationNodeSetElement.addContent(configurationNodeElement);
        configurationNodeSet=simpleXmlSerializer.deserializeConfigurationNodeSet(configurationNodeSetElement);
        assertCollectionContainsExactly(configurationNodeSet, new ConfigurationNodeImpl("MyPredicate"));

        // two element set
        Element configurationNodeElement2=new Element(SimpleXmlSerializer.CONFIGURATION_NODE_ELEMENT);
        configurationNodeElement2.setAttribute(SimpleXmlSerializer.PREDICATE_LABEL_ATTRIBUTE, "MyPredicate2");
        configurationNodeSetElement.addContent(configurationNodeElement2);
        configurationNodeSet=simpleXmlSerializer.deserializeConfigurationNodeSet(configurationNodeSetElement);
        assertCollectionContainsExactly(configurationNodeSet, new ConfigurationNodeImpl("MyPredicate"), new ConfigurationNodeImpl("MyPredicate"));
    }

    //----------------------------------------------------------------
    public void testDeserializeTaskNode() throws XmlDeserializationException {
        SimpleXmlSerializer simpleXmlSerializer=new SimpleXmlSerializer();

        // null input param
        try {
            simpleXmlSerializer.deserializeTaskNode(null);
            fail("Expected null root element to fail.");
        } catch (RequirementFailure e) {
            assertTrue(e.isThisStackFrameCulprit(0));
        }

        // wrong element name
        Element wrongElement=new Element("Wrong");
        try {
            simpleXmlSerializer.deserializeTaskNode(wrongElement);
            fail("Expected bad name to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Expected \"Task\" element, but got \"Wrong\" element.", e.getMessage());
        }

        ConfigurationNode configurationNodeA=new ConfigurationNodeImpl("A");
        Element configurationNodeElementA=new Element(SimpleXmlSerializer.CONFIGURATION_NODE_ELEMENT);
        configurationNodeElementA.setAttribute(SimpleXmlSerializer.PREDICATE_LABEL_ATTRIBUTE, "A");

        Element preconditionElement=new Element(SimpleXmlSerializer.PRECONDITION_ELEMENT);
        preconditionElement.addContent(configurationNodeElementA);

        ConfigurationNode configurationNodeB=new ConfigurationNodeImpl("B");
        Element configurationNodeElementB=new Element(SimpleXmlSerializer.CONFIGURATION_NODE_ELEMENT);
        configurationNodeElementB.setAttribute(SimpleXmlSerializer.PREDICATE_LABEL_ATTRIBUTE, "B");

        Element postconditionElement=new Element(SimpleXmlSerializer.POSTCONDITION_ELEMENT);
        postconditionElement.addContent(configurationNodeElementB);

        Element taskNodeElement=new Element(SimpleXmlSerializer.TASK_NODE_ELEMENT);
        taskNodeElement.setAttribute(SimpleXmlSerializer.CONTRACT_LABEL_ATTRIBUTE, "MyContract");
        taskNodeElement.setAttribute(SimpleXmlSerializer.JUNCTION_ATTRIBUTE, SimpleXmlSerializer.CONJUNCT_VALUE);
        taskNodeElement.addContent(preconditionElement);
        taskNodeElement.addContent(postconditionElement);

        // normal
        TaskNode taskNode=simpleXmlSerializer.deserializeTaskNode(taskNodeElement);
        assertEquals(new TaskNodeImpl("MyContract", Junction.CONJUNCT, CollectionUtil.setFromArray(configurationNodeA), CollectionUtil.setFromArray(configurationNodeB)), taskNode);

        // missing contract label attribute
        taskNodeElement.removeAttribute(SimpleXmlSerializer.CONTRACT_LABEL_ATTRIBUTE);
        try {
            simpleXmlSerializer.deserializeTaskNode(taskNodeElement);
            fail("Expected missing attribute to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("\"Task\" element is missing \"ContractLabel\" attribute.", e.getMessage());
        }
        taskNodeElement.setAttribute(SimpleXmlSerializer.CONTRACT_LABEL_ATTRIBUTE, "MyContract");

        // missing junction attribute
        taskNodeElement.removeAttribute(SimpleXmlSerializer.JUNCTION_ATTRIBUTE);
        try {
            simpleXmlSerializer.deserializeTaskNode(taskNodeElement);
            fail("Expected missing attribute to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("\"Task\" element is missing \"Junction\" attribute.", e.getMessage());
        }
        taskNodeElement.setAttribute(SimpleXmlSerializer.JUNCTION_ATTRIBUTE, SimpleXmlSerializer.CONJUNCT_VALUE);

        // disjuct junction attribute
        taskNodeElement.setAttribute(SimpleXmlSerializer.JUNCTION_ATTRIBUTE, SimpleXmlSerializer.DISJUNCT_VALUE);
        taskNode=simpleXmlSerializer.deserializeTaskNode(taskNodeElement);
        assertEquals(new TaskNodeImpl("MyContract", Junction.DISJUNCT, CollectionUtil.setFromArray(configurationNodeA), CollectionUtil.setFromArray(configurationNodeB)), taskNode);

        // unknown junction attribute
        taskNodeElement.setAttribute(SimpleXmlSerializer.JUNCTION_ATTRIBUTE, "strange");
        try {
            simpleXmlSerializer.deserializeTaskNode(taskNodeElement);
            fail("Expected bad attribute value to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Expected \"*\" or \"+\" for \"Junction\" attribute, but got \"strange\".", e.getMessage());
        }
        taskNodeElement.setAttribute(SimpleXmlSerializer.JUNCTION_ATTRIBUTE, SimpleXmlSerializer.CONJUNCT_VALUE);

        // unknown content element
        taskNodeElement.addContent(wrongElement);
        try {
            simpleXmlSerializer.deserializeTaskNode(taskNodeElement);
            fail("Expected unknown content element to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Unknown child element \"Wrong\" in \"Task\" element.", e.getMessage());
        }
        taskNodeElement.removeContent(wrongElement);

        // missing precondition
        taskNodeElement.removeContent(preconditionElement);
        try {
            simpleXmlSerializer.deserializeTaskNode(taskNodeElement);
            fail("Expected missing precondition element to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("No precondition configuration nodes specified.", e.getMessage());
        }
        taskNodeElement.addContent(preconditionElement);

        // missing postcondition
        taskNodeElement.removeContent(postconditionElement);
        try {
            simpleXmlSerializer.deserializeTaskNode(taskNodeElement);
            fail("Expected missing postcondition element to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("No postcondition configuration nodes specified.", e.getMessage());
        }
        taskNodeElement.addContent(postconditionElement);
    }

    //----------------------------------------------------------------
    public void testDeserializeCompositeWorkflow() throws XmlDeserializationException {
        SimpleXmlSerializer simpleXmlSerializer=new SimpleXmlSerializer();

        // null input param
        try {
            simpleXmlSerializer.deserializeCompositeWorkflow(null);
            fail("Expected null root element to fail.");
        } catch (RequirementFailure e) {
            assertTrue(e.isThisStackFrameCulprit(0));
        }

        // wrong element name
        Element wrongElement=new Element("Wrong");
        try {
            simpleXmlSerializer.deserializeCompositeWorkflow(wrongElement);
            fail("Expected bad name to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Expected \"CompositeWorkflow\" element, but got \"Wrong\" element.", e.getMessage());
        }

        // empty workflow
        Element compositeWorkflowElement=new Element(SimpleXmlSerializer.COMPOSITE_WORKFLOW_ELEMENT);
        CompositeWorkflow compositeWorkflow=simpleXmlSerializer.deserializeCompositeWorkflow(compositeWorkflowElement);
        assertCollectionContainsExactly(compositeWorkflow.getTaskNodes());

        // unknown content element
        compositeWorkflowElement.addContent(wrongElement);
        try {
            simpleXmlSerializer.deserializeCompositeWorkflow(compositeWorkflowElement);
            fail("Expected unknown content element to fail.");
        } catch (XmlDeserializationException e) {
            assertEquals("Unknown child element \"Wrong\" in \"CompositeWorkflow\" element.", e.getMessage());
        }
        compositeWorkflowElement.removeContent(wrongElement);

        ConfigurationNode configurationNodeA=new ConfigurationNodeImpl("A");
        Element configurationNodeElementA=new Element(SimpleXmlSerializer.CONFIGURATION_NODE_ELEMENT);
        configurationNodeElementA.setAttribute(SimpleXmlSerializer.PREDICATE_LABEL_ATTRIBUTE, "A");

        Element preconditionElement=new Element(SimpleXmlSerializer.PRECONDITION_ELEMENT);
        preconditionElement.addContent(configurationNodeElementA);

        ConfigurationNode configurationNodeB=new ConfigurationNodeImpl("B");
        Element configurationNodeElementB=new Element(SimpleXmlSerializer.CONFIGURATION_NODE_ELEMENT);
        configurationNodeElementB.setAttribute(SimpleXmlSerializer.PREDICATE_LABEL_ATTRIBUTE, "B");

        Element postconditionElement=new Element(SimpleXmlSerializer.POSTCONDITION_ELEMENT);
        postconditionElement.addContent(configurationNodeElementB);

        TaskNode taskNode=new TaskNodeImpl("MyContract", Junction.CONJUNCT, CollectionUtil.setFromArray(configurationNodeA), CollectionUtil.setFromArray(configurationNodeB));
        Element taskNodeElement=new Element(SimpleXmlSerializer.TASK_NODE_ELEMENT);
        taskNodeElement.setAttribute(SimpleXmlSerializer.CONTRACT_LABEL_ATTRIBUTE, "MyContract");
        taskNodeElement.setAttribute(SimpleXmlSerializer.JUNCTION_ATTRIBUTE, SimpleXmlSerializer.CONJUNCT_VALUE);
        taskNodeElement.addContent(preconditionElement);
        taskNodeElement.addContent(postconditionElement);

        // one task
        compositeWorkflowElement.addContent(taskNodeElement);
        compositeWorkflow=simpleXmlSerializer.deserializeCompositeWorkflow(compositeWorkflowElement);
        assertCollectionContainsExactly(compositeWorkflow.getTaskNodes(), taskNode);


        ConfigurationNode configurationNodeC=new ConfigurationNodeImpl("C");
        Element configurationNodeElementC=new Element(SimpleXmlSerializer.CONFIGURATION_NODE_ELEMENT);
        configurationNodeElementC.setAttribute(SimpleXmlSerializer.PREDICATE_LABEL_ATTRIBUTE, "C");

        Element preconditionElement2=new Element(SimpleXmlSerializer.PRECONDITION_ELEMENT);
        preconditionElement2.addContent(configurationNodeElementC);

        ConfigurationNode configurationNodeD=new ConfigurationNodeImpl("D");
        Element configurationNodeElementD=new Element(SimpleXmlSerializer.CONFIGURATION_NODE_ELEMENT);
        configurationNodeElementD.setAttribute(SimpleXmlSerializer.PREDICATE_LABEL_ATTRIBUTE, "D");

        Element postconditionElement2=new Element(SimpleXmlSerializer.POSTCONDITION_ELEMENT);
        postconditionElement2.addContent(configurationNodeElementD);

        TaskNode taskNode2=new TaskNodeImpl("MyContract2", Junction.DISJUNCT, CollectionUtil.setFromArray(configurationNodeC), CollectionUtil.setFromArray(configurationNodeD));
        Element taskNodeElement2=new Element(SimpleXmlSerializer.TASK_NODE_ELEMENT);
        taskNodeElement2.setAttribute(SimpleXmlSerializer.CONTRACT_LABEL_ATTRIBUTE, "MyContract2");
        taskNodeElement2.setAttribute(SimpleXmlSerializer.JUNCTION_ATTRIBUTE, SimpleXmlSerializer.DISJUNCT_VALUE);
        taskNodeElement2.addContent(preconditionElement2);
        taskNodeElement2.addContent(postconditionElement2);

        // two tasks
        compositeWorkflowElement.addContent(taskNodeElement2);
        compositeWorkflow=simpleXmlSerializer.deserializeCompositeWorkflow(compositeWorkflowElement);
        assertCollectionContainsExactly(compositeWorkflow.getTaskNodes(), taskNode, taskNode2);
    }
}