package edu.wustl.cse.mobilab.openworkflow.workflow;

import edu.wustl.cse.mobilab.openworkflow.comm.FragmentQueryable;
import edu.wustl.cse.mobilab.openworkflow.shell.InMemoryPersistenceProvider;
import edu.wustl.cse.mobilab.openworkflow.util.BaseTestCase;
import edu.wustl.cse.mobilab.openworkflow.util.RecordingMockObject;
import edu.wustl.cse.mobilab.openworkflow.util.SimpleXmlSerializer;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.Collections;

//--------------------------------------------------------------------
/** Tests for {@link edu.wustl.cse.mobilab.openworkflow.workflow.SetBetaFunction}
 * @author Created by Louis Thomas on Jan 15, 2009. */
public class TestFragmentManagerImpl extends BaseTestCase {

    private LoggingRecorder m_loggingRecorder;

    //----------------------------------------------------------------
    @Override
    protected void setUp() {
        m_loggingRecorder=new LoggingRecorder(Logger.getLogger(FragmentManagerImpl.class));
    }

    //----------------------------------------------------------------
    @Override
    protected void tearDown() {
        m_loggingRecorder.detatch();
    }

    //----------------------------------------------------------------
    public void testFragmentManager() {

        ConfigurationNode configurationNodeA = new ConfigurationNodeImpl("A");
        ConfigurationNode configurationNodeC = new ConfigurationNodeImpl("C");
        TaskNode taskNode = new TaskNodeImpl("B", Junction.DISJUNCT, Collections.<ConfigurationNode>singleton(configurationNodeA), Collections.<ConfigurationNode>singleton(configurationNodeC));
        CompositeWorkflowBuilder compositeWorkflowBuilder = new CompositeWorkflowBuilderImpl();
        compositeWorkflowBuilder.addTaskNode(taskNode);

        SimpleXmlSerializer simpleXmlSerializer = new SimpleXmlSerializer();
        Element compositeWorkflowElement = simpleXmlSerializer.serializeCompositeWorkflow(compositeWorkflowBuilder);

        Element rootElement = new Element("Fragments");
        rootElement.addContent(compositeWorkflowElement);

        InMemoryPersistenceProvider inMemoryPersistenceProvider = new InMemoryPersistenceProvider();
        inMemoryPersistenceProvider.saveXmlByName(FragmentManager.FRAGMENTSTORE_NAME, rootElement);

        FragmentManager fragmentManager=new FragmentManagerImpl(inMemoryPersistenceProvider, Logger.getLogger(FragmentManagerImpl.class));

        MockFragmentQueryableListener mockFragmentQueryableListener = new MockFragmentQueryableListener();

        // find any fragments containing configuration "A"
        fragmentManager.subscribeFragmentsWithConfiguration(configurationNodeA, mockFragmentQueryableListener);
        assertEquals("handleFragment({CompositeWorkflowBuilder [{TaskNode contractLabel:\"B\" junction:DISJUNCT precondition:[{ConfigurationNode predicateLabel:\"A\"}] postcondition:[{ConfigurationNode predicateLabel:\"C\"}]}]})", mockFragmentQueryableListener.getActivityRecordAndReset());
        m_loggingRecorder.assertMessagesLogged(new CheckedMessage("FragmentManagerImpl: Subscription added for {ConfigurationNode predicateLabel:\"A\"}.", Level.INFO),
            new CheckedMessage("FragmentManagerImpl: {ConfigurationNode predicateLabel:\"A\"} found in {CompositeWorkflowBuilder [{TaskNode contractLabel:\"B\" junction:DISJUNCT precondition:[{ConfigurationNode predicateLabel:\"A\"}] postcondition:[{ConfigurationNode predicateLabel:\"C\"}]}]}.", Level.DEBUG));
    }

    //----------------------------------------------------------------
    private static class MockFragmentQueryableListener extends RecordingMockObject implements FragmentQueryable.Listener {
        public void handleFragment(CompositeWorkflow compositeWorkflow) {
            recordActivity("handleFragment("+compositeWorkflow+")");
        }
    }
}