package edu.wustl.cse.mobilab.openworkflow.workflow;

import edu.wustl.cse.mobilab.openworkflow.util.BaseTestCase;
import edu.wustl.cse.mobilab.openworkflow.util.CollectionUtil;
import edu.wustl.cse.mobilab.openworkflow.verify.RequirementFailure;

import java.util.Set;

//--------------------------------------------------------------------
/** Tests for {@link SetAlphaFunction}
 * @author Created by Louis Thomas on Jan 15, 2009. */
public class TestSetAlphaFunction extends BaseTestCase {

    //----------------------------------------------------------------
    public void testSetAlphaFunction() {

        // check constructor
        try {
            //noinspection ResultOfObjectAllocationIgnored
            new SetAlphaFunction(null);
            fail("Expected bad param to fail.");
        } catch (RequirementFailure requirementFailure) {
            assertTrue(requirementFailure.isThisStackFrameCulprit(0));
        }

        // check constructor
        ConfigurationNode configurationNodeA=new ConfigurationNodeImpl("A");
        ConfigurationNode configurationNodeB=new ConfigurationNodeImpl("B");
        Set<ConfigurationNode> configurationNodes=CollectionUtil.setFromArray(configurationNodeB, configurationNodeA);
        SetAlphaFunction setAlphaFunction=new SetAlphaFunction(configurationNodes);

        // check toString
        assertEquals("{SetAlphaFunction [{ConfigurationNode predicateLabel:\"A\"}, {ConfigurationNode predicateLabel:\"B\"}]}", setAlphaFunction.toString());

        // check isolation
        ConfigurationNode configurationNodeC=new ConfigurationNodeImpl("C");
        configurationNodes.add(configurationNodeC);
        assertEquals("{SetAlphaFunction [{ConfigurationNode predicateLabel:\"A\"}, {ConfigurationNode predicateLabel:\"B\"}]}", setAlphaFunction.toString());


        // check isInitialConfiguration
        try {
            setAlphaFunction.isInitialConfiguration(null);
            fail("Expected bad param to fail.");
        } catch (RequirementFailure requirementFailure) {
            assertTrue(requirementFailure.isThisStackFrameCulprit(0));
        }

        // check isInitialConfiguration
        assertTrue(setAlphaFunction.isInitialConfiguration(configurationNodeA));
        assertTrue(setAlphaFunction.isInitialConfiguration(configurationNodeB));
        assertFalse(setAlphaFunction.isInitialConfiguration(configurationNodeC));
    }
}