package edu.wustl.cse.mobilab.openworkflow.workflow;

import edu.wustl.cse.mobilab.openworkflow.util.BaseTestCase;
import edu.wustl.cse.mobilab.openworkflow.util.CollectionUtil;
import edu.wustl.cse.mobilab.openworkflow.verify.RequirementFailure;

import java.util.Collections;
import java.util.Set;

//--------------------------------------------------------------------
/** Tests for {@link SetBetaFunction}
 * @author Created by Louis Thomas on Jan 15, 2009. */
public class TestSetBetaFunction extends BaseTestCase {

    //----------------------------------------------------------------
    public void testSetBetaFunction() {
        // check constructor
        try {
            //noinspection ResultOfObjectAllocationIgnored
            new SetBetaFunction(null);
            fail("Expected bad param to fail.");
        } catch (RequirementFailure requirementFailure) {
            assertTrue(requirementFailure.isThisStackFrameCulprit(0));
        }

        // check constructor
        ConfigurationNode configurationNodeA=new ConfigurationNodeImpl("A");
        ConfigurationNode configurationNodeB=new ConfigurationNodeImpl("B");
        Set<ConfigurationNode> configurationNodes=CollectionUtil.setFromArray(configurationNodeB, configurationNodeA);
        SetBetaFunction setBetaFunction=new SetBetaFunction(configurationNodes);

        // check toString
        assertEquals("{SetBetaFunction [{ConfigurationNode predicateLabel:\"A\"}, {ConfigurationNode predicateLabel:\"B\"}]}", setBetaFunction.toString());

        // check isolation
        ConfigurationNode configurationNodeC=new ConfigurationNodeImpl("C");
        configurationNodes.add(configurationNodeC);
        assertEquals("{SetBetaFunction [{ConfigurationNode predicateLabel:\"A\"}, {ConfigurationNode predicateLabel:\"B\"}]}", setBetaFunction.toString());


        // check identifyFinalConfigurations
        try {
            setBetaFunction.identifyFinalConfigurations(null);
            fail("Expected bad param to fail.");
        } catch (RequirementFailure requirementFailure) {
            assertTrue(requirementFailure.isThisStackFrameCulprit(0));
        }

        // check identifyFinalConfigurations
        assertNull(setBetaFunction.identifyFinalConfigurations(Collections.<ConfigurationNode>emptySet()));
        assertNull(setBetaFunction.identifyFinalConfigurations(CollectionUtil.setFromArray(configurationNodeA)));
        assertNull(setBetaFunction.identifyFinalConfigurations(CollectionUtil.setFromArray(configurationNodeA, configurationNodeC)));
        assertCollectionContainsExactly(setBetaFunction.identifyFinalConfigurations(CollectionUtil.setFromArray(configurationNodeA, configurationNodeB)), configurationNodeA, configurationNodeB);
        assertCollectionContainsExactly(setBetaFunction.identifyFinalConfigurations(CollectionUtil.setFromArray(configurationNodeB, configurationNodeC, configurationNodeA)), configurationNodeA, configurationNodeB);
    }
}